<?php

namespace App\Console\Commands;

use App\Jobs\StartEmailCheckFromGoogleAPI;
use App\Models\GoogleUser;
use App\Models\MillionVerifyJob;
use App\Models\VerifiedEmail;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Log\Logger;
use Illuminate\Support\Facades\Log;

use function Psy\debug;

class CheckEmailStatus extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'email:check-status';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Check and update the status of emails';

    /**
     * Execute the console command.
     */
    public function handle()
    {

        $timeToCheckForJob = Carbon::now()->subMinutes(5);

        $millionVerifyJobIds = MillionVerifyJob::
            where('created_at', '<=', $timeToCheckForJob)
            ->where('is_catch_all_processed', MillionVerifyJob::IS_CATCH_ALL_PROCESSED_PROCESSED)
            ->pluck('million_verify_job_id')
            //  ->where('created_at', '<=', $timeToCheckForJob)

            ->toArray();
        //$timeToCheck = Carbon::now()->subHours(3);
        $timeToCheck = Carbon::now()->subMinutes(20);

        Log::debug("time imd", [$millionVerifyJobIds]);

        foreach ($millionVerifyJobIds as $millionVerifyJobId) {
            // Getting all unprocessed emails
            $googleUserIds = VerifiedEmail::where('million_verify_job_id', $millionVerifyJobId)
                ->where('status', VerifiedEmail::PENDING_STATUS) // Pending To Check From Inbox
                ->where('is_sent', VerifiedEmail::SENT) // Email had been sent
                ->where('result', VerifiedEmail::RESULT_PENDING) // Result From Google Email is Pending
                ->where('sent_at', '<=', $timeToCheck)
                ->pluck('google_user_id')
                ->unique()
                ->toArray();
            Log::debug("Found " . implode($googleUserIds) . "count " . count($googleUserIds));
            if (count($googleUserIds) === 0) {
              $totalEmails =  VerifiedEmail::where('million_verify_job_id', $millionVerifyJobId)->count();
              if($totalEmails > 0){
               $totalPendingEmails = VerifiedEmail::where('million_verify_job_id', $millionVerifyJobId)->where('status', VerifiedEmail:: PENDING_STATUS)->count();
               if($totalPendingEmails === 0){
                  MillionVerifyJob::where('million_verify_job_id', $millionVerifyJobId)->update([
                    'is_catch_all_processed' => MillionVerifyJob::IS_CATCH_ALL_PROCESSED_COMPLETED
                ]);
               }

              }

            }
            if (count($googleUserIds) > 0) {

                $googleEmails = GoogleUser::whereIn('google_user_id', $googleUserIds)->get();
                Log::debug("Google User count " . count($googleEmails));
                foreach ($googleEmails as $googleEmail) {

                    $verifiedEmails = VerifiedEmail::where('million_verify_job_id', $millionVerifyJobId)
                        ->where('status', VerifiedEmail::PENDING_STATUS) // Pending To Check From Inbox
                        ->where('is_sent', VerifiedEmail::SENT) // Email had been sent
                        ->where('result', VerifiedEmail::RESULT_PENDING) // Result From Google Email is Pending
                        ->where('sent_at', '<=', $timeToCheck)
                        ->where('google_user_id', $googleEmail->google_user_id)
                        ->take(100)
                        ->get();
                    foreach ($verifiedEmails as $verifiedEmail) {
                        $verifiedEmail->result = VerifiedEmail::RESULT_PROCESSING;
                        $verifiedEmail->save();
                    }
                    StartEmailCheckFromGoogleAPI::dispatch($verifiedEmails, $googleEmail->google_user_id);
                }
            }
        }


        $this->info('Email status checked and updated successfully.');
    }
}
