<?php

namespace App\Console\Commands;

use App\Jobs\ProcessSendEmail;
use App\Jobs\SendEmail;
use App\Models\GoogleUser;
use App\Models\MillionVerifyJob;
use App\Models\VerifiedEmail;
use App\Services\MillionVerify\MillionVerify;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class PoolEmailProcessingCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'pool:processing';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Reset the mails_sent_in_a_day column for GoogleUser records';

    /**
     * Execute the console command.
     */
    public function handle()
    {

        $millionVerifyProcesses = MillionVerifyJob::
             select('million_verify_job.million_verify_job_id', 'million_verify_job.user_id') // Select all columns from million_verify_job
            ->where('is_catch_all_processed', MillionVerifyJob::IS_CATCH_ALL_PROCESSED_PROCESSED)
            //->take(2)
            ->join('verified_emails', 'million_verify_job.million_verify_job_id', '=', 'verified_emails.million_verify_job_id')
            ->groupBy('million_verify_job.million_verify_job_id', 'million_verify_job.user_id') // Include file_id in the GROUP BY clause
            //->where('verified_emails.is_sent', 0)
            ->havingRaw('COUNT(verified_emails.is_sent) > 0')
            ->get();
        Log::debug("--------------Total Million Verify To Send Email---------------" . count($millionVerifyProcesses));
        foreach ($millionVerifyProcesses as $millionVerify) {

            $verifiedEmails = VerifiedEmail::where('million_verify_job_id', $millionVerify->million_verify_job_id)
                ->where('status', VerifiedEmail::PENDING_STATUS) // Pending
                ->where('is_sent', VerifiedEmail::NOT_SENT) // Not Sent Email
                ->take(500) // Limiting to 500 records
                ->pluck('email')
                ->toArray();

            $totalEmails = count($verifiedEmails);
            Log::debug("-----------Total------------------" . $totalEmails);

            $emailIndex = 0; // Index to track the current email being sent
            $userIndex = 0; // Index to track the current user
            while ($emailIndex < $totalEmails) {
                $googleUsers = GoogleUser::select()
                    ->where('user_id', $millionVerify->user_id)
                    ->where('mails_sent_in_a_day', '<', 1500)
                    ->get();
                if (count($googleUsers) > 0) {
                    // Reset index if all users are iterated
                    $googleUser = $googleUsers[$userIndex % count($googleUsers)];

                    // Check if the current user has exceeded the limit
                    if ($googleUser->mails_sent_in_a_day >= 1500) {
                        // Remove the user from the list
                        unset($googleUsers[$userIndex % count($googleUsers)]);
                        $googleUsers = $googleUsers->values();

                        continue; // Skip to the next user
                    }
                    $emailBatch = [];
                    // Form email batch
                    $currentTimestamp = Carbon::now();
                    // For Tracking when we sent the email

                    for ($i = 0; $i < 15 && $emailIndex < $totalEmails; $i++) {
                        $emailBatch[] = $verifiedEmails[$emailIndex];

                       $emailound = VerifiedEmail::
                            where('email', $verifiedEmails[$emailIndex])
                            ->update([
                                'is_sent' => VerifiedEmail::SENT,
                                'sent_at' => $currentTimestamp,
                                'result' => VerifiedEmail::RESULT_PENDING,
                                'google_user_id' => $googleUser->google_user_id
                            ]);
                            Log::debug("Herre update the is_sent",[$emailound]);
                        $emailIndex++;
                        // Update Per Email Count
                        $googleUser->mails_sent_in_a_day += 1;
                        $googleUser->save();
                    }
                    Log::debug($googleUser->google_name . "\n");

                    ProcessSendEmail::dispatch($googleUser, $emailBatch);
                    //SendEmail::dispatch($googleUser, $emailBatch); //->delay(now()->addSeconds(30));
                    // SendEmail::dispatch($this->googleUser, $this->emailBatch)->delay(now()->addSeconds(30));

                    // Update mails_sent_in_a_day column for the current user by 1 because if we send 15 emails in bcc it counts it as one


                    $userIndex++; // Move to the next user
                } else {
                    break;
                }
            }
        }
    }
}
