<?php

namespace App\Console\Commands;

use App\Jobs\StartEmailProcess;
use App\Jobs\TestQueue;
use App\Models\MillionVerifyJob;
use App\Services\MillionVerify\MillionVerify;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class millionverifycheck extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'app:millionverifycheck';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Execute the console command.
     */

    private function formatCsvData(string $csvData)
    {

        // Split the CSV data into lines
        $lines = explode("\n", $csvData);

        // Initialize the array to store the structured data
        $data = [];

        // Process each line (except the header)
        foreach ($lines as $index => $line) {
            // Skip the header line
            // if ($index === 0) continue;

            // Parse the line using str_getcsv() function
            $rowData = str_getcsv($line);

            // Add the parsed data to the main array
            $data[] = $rowData;
        }
        return $data;
    }

    public function handle()
    {
        //
        $records = DB::table('million_verify_job')
            ->where('percent', '<', 100)
            ->where('is_catch_all_processed', MillionVerifyJob::IS_CATCH_ALL_PROCESSED_PENDING)
            ->get();
        $apiKey = env('MILLION_VERIFY');

        $resultFilesDownloader = new MillionVerify($apiKey);

        foreach ($records as $item) {
            // Log::debug($item->file_id);
            // Download the list of result files
            $resultFiles = $resultFilesDownloader->checkFileStatus($item->file_id);
            Log::debug("Result", $resultFiles);
            if ($resultFiles['percent'] === 100) {
                if ($resultFiles['catch_all'] > 0) {
                    // Log::debug("Found catch all email " . $resultFiles['catch_all'] . "Start proces");
                    $downloadFile = $resultFilesDownloader->downloadCSV($item->file_id);
                    $structuredData = $this->formatCsvData($downloadFile);
                    $fileName = 'data_' . time() . '.csv';
                    $file = fopen(storage_path('app/uploads/catchall/' . $fileName), 'w');
                    foreach ($structuredData as $row) {
                        fputcsv($file, $row);
                    }
                    fclose($file);
                    //  Log::debug($downloadFile);
                    StartEmailProcess::dispatch($fileName, $item);
                    DB::table('million_verify_job')
                        ->where('million_verify_job_id', $item->million_verify_job_id) // Specify the condition for the record you want to update
                        ->update([
                            'percent' => $resultFiles['percent'], // Update the 'status' column to 'completed'
                            'catch_all' => $resultFiles['catch_all'],
                            'is_catch_all_processed' => MillionVerifyJob::IS_CATCH_ALL_PROCESSED_PROCESSED   // Optionally, update the 'updated_at' column with the current timestamp
                        ]);
                } else {
                    DB::table('million_verify_job')
                        ->where('million_verify_job_id', $item->million_verify_job_id) // Specify the condition for the record you want to update
                        ->update([
                            'percent' => $resultFiles['percent'], // Update the 'status' column to 'completed'
                            'is_catch_all_processed' => MillionVerifyJob::IS_CATCH_ALL_PROCESSED_COMPLETED   // Optionally, update the 'updated_at' column with the current timestamp
                        ]);
                }
            } else {
                DB::table('million_verify_job')
                    ->where('million_verify_job_id', $item->million_verify_job_id) // Specify the condition for the record you want to update
                    ->update([
                        'percent' => $resultFiles['percent'], // Update the 'status' column to 'completed'
                    ]);
            }
        }
    }
}
