<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\User;
use Carbon\Carbon;
use Google\Service\ServiceControl\Auth;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth as FacadesAuth;
use Illuminate\Support\Facades\Hash;

class UserAuthController extends Controller
{
    CONST SUPER_ADMIN = 1;
    CONST ADMIN = 2;
    CONST USER = 3;
    public function register(Request $request)
    {

        $data = $request->validate([
            'name' => 'required|max:255',
            'email' => 'required|email|unique:users',
            'password' => 'required',
            'user_type' => 'integer',
        ]);

        // THIS IS INCORRECT LOGIC
        // $request['type'] = $request['type'] ? $request['type']  : 0;

        $data['user_type'] = ($request->has('user_type') && $request->type != null)
                            ? $request->type
                            : self::USER;

        $data['password'] = bcrypt($request->password);

        $user = User::create($data);

        $token = $user->createToken('API Token')->accessToken;

        return response(['user' => $user, 'token' => $token]);
    }

    public function login(Request $request)
    {
        $request->validate([
            'email'         => 'email|required',
            'password'      => 'required',
        ]);
        $user = null;
        if(isset($request->is_admin) && $request->is_admin === true ){
            $user = User::where('email', $request->email)->where('user_type', self::SUPER_ADMIN)->first();
        }
        else{
            $user = User::where('email', $request->email)->first();
        }



        // FOR EACH RESPECTIVE PRODUCT WE HAVE DIFFERENT USER IDS
        // $leadUserTypesArray = [1,2];
        // $emailUserTypesArray = [3,4];

        // $isAuthorized = false;

        // if($productId === null && $user->user_type === 0) {
        //     $isAuthorized = true;
        // } else if (in_array($user->type, $leadUserTypesArray) && $productId == 1) {
        //     $isAuthorized = true;
        // } elseif (in_array($user->type, $emailUserTypesArray) && $productId == 2) {
        //     $isAuthorized = true;
        // }

        // LOGIC IN PROGRESS


        if ($user) {
            if (Hash::check($request->password, $user->password)) {
                $accessToken = $user->createToken('API-TOKEN')->accessToken;
                // return $this->respondWithToken($request->user()->refreshToken());


                if ($user->user_type === self::USER) {

                    $user->load("subscriptions");

                    // TEST LOGIC
                    $productPrefix = $request->header('product-prefix');

                    if (!$productPrefix) {
                        return response()->json(['success' => false, 'message' => 'Product prefix not provided'], 400);
                    }

                    $product = Product::where('prefix', $productPrefix)->first();

                    if (!$product) {
                        return response()->json(['success' => false, 'message' => 'Product not found'], 404);
                    }

                    $userWallets = $user->user_wallets()->where('product_id', $product->product_id)->first();
                    $user->setRelation('user_wallets', $userWallets);
                    // TEST LOGIC END
                }


                $response = ['accessToken' => $accessToken,  'userDetail' => $user];
                return response($response, 200);
            } else {
                $response = ["message" => "Password mismatch"];
                return response($response, 422);
            }
        } else {
            $response = ["message" => 'User does not exist'];
            return response($response, 422);
        }
    }

    public function checkemail(Request $request)
    {

        $data = $request->validate([
            'email' => 'email|required',
        ]);
        $user = User::where('email', $request->email)->first();
        if ($user) {

            $response = ["exist" => true];
            return response($response, 200);
        } else {
            $response = ["exist" => false];
            return response($response, 200);
        }
    }

    protected function respondWithToken($token)
    {
        return response()->json([
            'success' => true,
            'accessToken' => $token,
            'token_type' => 'Bearer',

        ]);
    }

    public function me_old(Request $request)
    {
        if ($request->user()) {
            $user = $request->user();

            $user->load("subscriptions","user_wallets");

            $response = ['userDetail' => $user];
            return response($response, 200);
        } else {
            return response()->json(['success' => false, 'message' => 'User is not authenticated']);
        }
    }

    public function me(Request $request)
    {
        // Check if user is authenticated
        if (!$request->user()) {
            return response()->json(['success' => false, 'message' => 'User is not authenticated'], 401);
        }

        $user = $request->user();

        // TEST LOGIC
        $productPrefix = $request->header('Product-Prefix');

        if($user->user_type === self::USER) {

            if (!$productPrefix) {
                return response()->json(['success' => false, 'message' => 'Product prefix not provided'], 400);
            }

            $product = Product::where('prefix', $productPrefix)->first();

            if (!$product) {
                return response()->json(['success' => false, 'message' => 'Product not found'], 404);
            }

            $user->load('subscriptions');
            $userWallets = $user->user_wallets()->where('product_id', $product->product_id)->first();
            $user->setRelation('user_wallets', $userWallets);

        }
        // TEST LOGIC END

        $response = ['userDetail' => $user];
        return response($response, 200);
    }



    public function refreshToken(Request $request)
    {
        if ($request->user()) {
            $user = $request->user();
            $user->tokens()->delete();
            $user->load("subscriptions","user_wallets");
            $accessToken = $user->createToken('API-TOKEN')->accessToken;
            // return $this->respondWithToken($request->user()->refreshToken());

            $response = ['accessToken' => $accessToken,  'userDetail' => $user];
            return response($response, 200);
        } else {
            return response()->json(['success' => false, 'message' => 'User is not authenticated']);
        }
    }

    protected function sendResetLinkResponse(Request $request, $response)
    {
        $response = ['message' => "Password reset email sent"];
        return response($response, 200);
    }
    protected function sendResetLinkFailedResponse(Request $request, $response)
    {
        $response = "Email could not be sent to this email address";
        return response($response, 500);
    }

    protected function resetPassword($user, $password)
    {
        $user->password = Hash::make($password);
        $user->save();
        event(new PasswordReset($user));
    }
    protected function sendResetResponse(Request $request, $response)
    {
        $response = ['message' => "Password reset successful"];
        return response($response, 200);
    }
    protected function sendResetFailedResponse(Request $request, $response)
    {
        $response = "Token Invalid";
        return response($response, 401);
    }
}
