<?php

namespace App\Http\Controllers\CloudLead;

use App\Http\Controllers\Controller;
use App\Models\CustomLeadRequest;
use App\Traits\PaginateTrait;
use Illuminate\Support\Facades\Storage;
use Illuminate\Http\Request;

class CustomLeadRequestController extends Controller
{
    use PaginateTrait;

    public function store(Request $request)
    {
        $request->validate([
            'first_name' => 'required',
            'last_name' => 'required',
            // Add other validation rules as needed
        ]);
        $csv_file_url = "";
        if ($request->file('csv_file')) {
            $csv_file = $request->file('csv_file')->store('csvs', 'public');
            $csv_file_url = Storage::url($csv_file);
        }


        $leadRequest = CustomLeadRequest::create([
            'first_name' => $request->first_name,
            'last_name' => $request->last_name,
            'linkedin_url' => $request->linkedin_url,
            'company_name' => $request->company_name,
            'industry' => $request->industry,
            'website_url' => $request->website_url,
            'country' => $request->country,
            'city' => $request->city,
            'state' => $request->state,
            'zip_code' => $request->zip_code,
            'employee_size' => $request->employee_size,
            'job_title' => $request->job_title,
            'job_description' => $request->job_description,
            'contact_location' => $request->contact_location,
            'csv_file' => $csv_file_url,
            'user_id' =>  $request->user()->id,  // Assuming the user is authenticated
            'status' => 'PENDING', // Default status
        ]);

        return response()->json($leadRequest, 201);
    }

    public function index(Request $request)
    {
        $input = $request->all();

        $this->setDefaultPaginationValues(input: $input);
        $data = $this->paginatedData();

        return response()->json($data);
    }

    private function paginatedData()
    {
        $query =  CustomLeadRequest::select();
        if (isset($this->searchFilter) && !empty($this->searchFilter)) {
            $query->where('feature_name', 'like', '%' . $this->searchFilter . '%');
        }

        if (isset($this->sortingColumns) && is_array($this->sortingColumns) && count($this->sortingColumns) > 1 && (!empty($this->sortingColumns[0]) && !empty($this->sortingColumns[1]))) {
            $query->orderBy($this->sortingColumns[0], $this->sortingColumns[1]);
        }

        $data = $query->paginate($this->perPage, NULL, NULL, $this->page);

        return isset($data) ? $data->toArray() : [];
    }

    public function show(CustomLeadRequest $leadRequest)
    {
        return response()->json($leadRequest);
    }

    public function update(Request $request, CustomLeadRequest $leadRequest)
    {
        $request->validate([
            'first_name' => 'required',
            'last_name' => 'required',
            'user_id' => 'required|exists:users,id', // Validate user_id exists in users table
            // Add other validation rules as needed
        ]);

        $leadRequest->update($request->all());

        return response()->json($leadRequest);
    }

    public function destroy(CustomLeadRequest $leadRequest)
    {
        $leadRequest->delete();

        return response()->json(null, 204);
    }
}
