<?php

namespace App\Http\Controllers\CloudLead;
use App\Http\Controllers\Controller;
use App\Models\CustomLeadRequest;
use App\Models\CustomLeadRequestResponse;
use App\Traits\PaginateTrait;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class CustomLeadRequestResponseController extends Controller
{
    use PaginateTrait;

    public function store(Request $request)
    {
        $request->validate([
            'csv_file' => 'required|file|mimes:csv,txt',
            'lead_request_id' => 'required|integer|max:255',
        ]);

        $file = $request->file('csv_file')->store('csvs','public');
        $file_url = Storage::url($file);

        CustomLeadRequest::where('lead_request_id', $request->lead_request_id)
        ->update(['status' => 'SUBMITTED']);

        $enrichment = CustomLeadRequestResponse::create([
            'csv_file' => $file_url,
            'lead_request_id' => $request->lead_request_id,
        ]);

        // Here, you can add logic to process the CSV files and perform enrichment

        return response()->json($enrichment, 201);
    }

    public function index(Request $request)
    {
        $input = $request->all();

        $this->setDefaultPaginationValues(input: $input);
        $data = $this->paginatedData();

        return response()->json($data);
    }

    private function paginatedData()
    {
        $query =  CustomLeadRequestResponse::select();
        if (isset($this->searchFilter) && !empty($this->searchFilter)) {
            $query->where('feature_name', 'like', '%' . $this->searchFilter . '%');
        }

        if (isset($this->sortingColumns) && is_array($this->sortingColumns) && count($this->sortingColumns) > 1 && (!empty($this->sortingColumns[0]) && !empty($this->sortingColumns[1]))) {
            $query->orderBy($this->sortingColumns[0], $this->sortingColumns[1]);
        }

        $data = $query->paginate($this->perPage, NULL, NULL, $this->page);

        return isset($data) ? $data->toArray() : [];
    }
}
