<?php

namespace App\Http\Controllers\CloudLead;

use App\Http\Controllers\Controller;
use App\Models\Enrichment;
use App\Traits\PaginateTrait;
use Google\Service\ServiceControl\Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class EnrichmentController extends Controller
{
    use PaginateTrait;

    //
    public function store(Request $request)
    {
        $request->validate([
            'linkedin_csv' => 'required|file|mimes:csv,txt',
            'company_csv' => 'required|file|mimes:csv,txt',
            'enrichment_type' => 'required|string|max:255',
        ]);

        $linkedinCsvPath = $request->file('linkedin_csv')->store('csvs','public');
        $companyCsvPath = $request->file('company_csv')->store('csvs','public');

        $linkedinCsvPath_url = Storage::url($linkedinCsvPath);
        $companyCsvPath_url = Storage::url($companyCsvPath);


        $enrichment = Enrichment::create([
            'linkedin_csv' => $linkedinCsvPath_url,
            'company_csv' => $companyCsvPath_url,
            'enrichment_type' => $request->enrichment_type,
            'user_id' =>  $request->user()->id,  // Assuming the user is authenticated
            'status' => 'PENDING',
        ]);

        // Here, you can add logic to process the CSV files and perform enrichment

        return response()->json($enrichment, 201);
    }

    public function index(Request $request)
    {
        $input = $request->all();

        $this->setDefaultPaginationValues(input: $input);
        $data = $this->paginatedData();

        return response()->json($data);
    }

    private function paginatedData()
    {
        $query =  Enrichment::select();
        if (isset($this->searchFilter) && !empty($this->searchFilter)) {
            $query->where('feature_name', 'like', '%' . $this->searchFilter . '%');
        }

        if (isset($this->sortingColumns) && is_array($this->sortingColumns) && count($this->sortingColumns) > 1 && (!empty($this->sortingColumns[0]) && !empty($this->sortingColumns[1]))) {
            $query->orderBy($this->sortingColumns[0], $this->sortingColumns[1]);
        }

        $data = $query->paginate($this->perPage, NULL, NULL, $this->page);

        return isset($data) ? $data->toArray() : [];
    }


    /**
     * Display the specified enrichment.
     *
     * @param  \App\Models\Enrichment  $enrichment
     * @return \Illuminate\Http\Response
     */
    public function show(Enrichment $enrichment)
    {
        return response()->json($enrichment);
    }
}
