<?php

namespace App\Http\Controllers\Google;

use App\Models\GoogleUser;
use App\Providers\RouteServiceProvider;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Redirect;
use Laravel\Socialite\Facades\Socialite;
use App\Http\Controllers\Controller;
use App\Jobs\PubSubEmailNotification;
use App\Models\UserLinkedGoogleAccounts;
use Carbon\Carbon;
use Exception;
use Google_Client;
use Google_Service;
use Google_Service_Gmail;
use Google_Service_Oauth2;
use Google_Service_Gmail_Message;
use Google_Service_Gmail_MessagePart;
use Google_Service_Gmail_MessagePayload;
use Google_Service_Gmail_MessagePartHeader;
use Google\Service\Gmail;
use Google\Service\Gmail\Message;
use Google\Service\Gmail\MessagePartHeader;
use Illuminate\Log\Logger;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use PDO;

class GoogleLoginController extends Controller
{

    public function redirectToGoogleNew()
    {
        $client = new Google_Client();
        $client->setClientId(env('GOOGLE_CLIENT_ID'));
        $client->setClientSecret(env('GOOGLE_CLIENT_SECRET'));
        $client->setRedirectUri(env('GOOGLE_REDIRECT'));
        $client->setAccessType('offline');
        // $client->setApprovalPrompt('force'); // Add this line

        $client->setScopes(
            [
                'openid',
                'profile',
                'email',
                // 'https://www.googleapis.com/auth/gmail.send',
                // 'https://www.googleapis.com/auth/gmail.readonly',
                // 'https://www.googleapis.com/auth/gmail.modify',
                // 'https://www.googleapis.com/auth/gmail.labels',
                // 'https://www.googleapis.com/auth/gmail.compose',
                // 'https://www.googleapis.com/auth/userinfo.email',
                // 'https://www.googleapis.com/auth/userinfo.profile',
                'https://mail.google.com/',
                'https://www.googleapis.com/auth/gmail.send',
            //    'https://www.googleapis.com/auth/gmail.metadata',
                // 'https://www.googleapis.com/auth/gmail.readonly',
                'https://www.googleapis.com/auth/gmail.modify',
                'https://www.googleapis.com/auth/gmail.labels',
            ]
        );
        // OLD RETURN
        // return redirect($client->createAuthUrl());
        // OLD RETURN END
        return $client->createAuthUrl();
    }
    private function updateAccessTokenInDatabase($newAccessToken, $googleUser)
    {
        // Assuming you have a model to represent your Google user
        // Update the access token in your database
        $user = GoogleUser::where('google_user_id', $googleUser->google_user_id)->first();
        if ($user) {
            $user->google_token = $newAccessToken['access_token'];
            if (isset($newAccessToken['refresh_token'])) {
                $user->refresh_token = $newAccessToken['refresh_token'];
            }
            $user->save();
        }
    }
    public function checkAccessTokenExpirationAndRefreshIfNeeded($accessToken, $refreshToken, $googleUser)
    {
        $client = new Google_Client();
        $client->setClientId(env('GOOGLE_CLIENT_ID'));
        $client->setClientSecret(env('GOOGLE_CLIENT_SECRET'));
        $client->setRedirectUri(env('GOOGLE_REDIRECT'));
        $client->setAccessType('offline');
        $client->setAccessToken($accessToken);

        // Check if the access token is expired
        if ($client->isAccessTokenExpired()) {
            // Set the refresh token
            $client->refreshToken($refreshToken);
            // Get the new access token
            $newAccessToken = $client->getAccessToken();

            // Store the new access token in your database
            // Replace the old access token with the new one in your database
            // This step depends on how you're managing tokens in your application
            $this->updateAccessTokenInDatabase($newAccessToken, $googleUser);

            return $newAccessToken['access_token'];
        }

        // Access token is still valid, no need to refresh
        return $accessToken;
    }

    public function handleGoogleCallback(Request $request)
    {
        try {
            $client = new Google_Client();
            $client->setClientId(env('GOOGLE_CLIENT_ID'));
            $client->setClientSecret(env('GOOGLE_CLIENT_SECRET'));
            $client->setRedirectUri(env('GOOGLE_REDIRECT'));
            $client->fetchAccessTokenWithAuthCode($request->get('code'));
            // Store or use the access token for later use
            $accessToken = $client->getAccessToken();
            $objOAuthService = new Google_Service_Oauth2($client);
            Log::debug("Handle Google callback ", [$accessToken]);
            if (isset($accessToken)) {
                $userData = $objOAuthService->userinfo->get();
                $_SESSION['access_token'] = $client->getAccessToken();
                $email = $userData->getEmail();
                $givenName = $userData->getGivenName();
                $lastHistoryId = $this->getLastHistoryId($accessToken['access_token']);
                $user = auth()->user();
                $reqUser = request()->user();
                Log::debug("user ", [$user]);
                Log::debug("reqUser ", [$reqUser]);


                // Use the existing refresh token if not provided in the response
                $refreshToken = null;

                if (isset($accessToken['refresh_token'])) {
                    $refreshToken = $accessToken['refresh_token'];
                }

                // Ensure you handle the case where the refresh token might be missing
                $refreshToken = $refreshToken ?? 'none';

                $googleUser = GoogleUser::updateOrInsert([
                    'google_id' => $userData->getId(),
                    'google_email' => $email,
                ], [
                    'auth_code' => $request->get('code'),
                    'user_id' => 1,
                    'google_name' =>  $givenName,
                    'google_picture' => $userData->getPicture(),
                    'google_token' => $accessToken['access_token'],
                    'refresh_token' => $refreshToken,
                    'google_token_expires_in' => $accessToken['expires_in'],
                    'meta_data' => json_encode($accessToken),
                    'is_active' => 0,
                    'last_history_id' => $lastHistoryId
                ]);
                // $this->setUpGmailPushNotifications($userData->getId());
            }

            // OLD RETURN
            // $response = [
            //     "message" => 'Success', 'google_token' => $accessToken['access_token'],
            //     'refresh_token' => $accessToken['refresh_token'],
            // ];
            // return response($response, 200);
            // OLD RETURN END

                    // Redirect to frontend URL with tokens as query parameters
        $redirectUrl = env('APP_URL') . '/success-integration';
        $redirectUrl .= '?access_token=' . $accessToken['access_token'];
        $redirectUrl .= '&refresh_token=' . $refreshToken;
        $redirectUrl .= '&email=' . $email;

        return redirect($redirectUrl);

        }
        catch(\Exception $ex){
            return response(['message'=>$ex->getMessage()], 401);

        }

    }

    public function redirectToGoogle()
    {
        return Socialite::driver('google')->redirect();
    }
    public function sendEmail($accessToken, $bccRecipients)
    {
        // Instantiate Google Client
        $client = new Google_Client();
        $client->setAccessToken($accessToken);

        // Create Gmail service object
        $service = new Gmail($client);

        // Create a new Gmail message
        $email = new Message();

        // Construct the email content
        $emailContent = [
            // 'From: sender@example.com',
            // 'To: ' . $toEmail,
            'Subject: Meeting',
            'Bcc: ' . implode(',', $bccRecipients), // Combine all Bcc recipients
            '', // Separate headers from the message body
            'Hello'
        ];
        $emailRaw = implode("\r\n", $emailContent);

        // Concatenate the email content lines with line breaks

        // Set the raw email content to the Gmail message
        $email->setRaw(base64_encode($emailRaw));

        // Add Bcc recipients


        Log::debug($emailRaw);
        // Send the email using the Gmail API
        $service->users_messages->send("me", $email);
    }

    public function sendEmail1($accessToken, $bccRecipients, $emailBody)
{
    try {
        // Instantiate Google Client
        $client = new Google_Client();
        $client->setAccessToken($accessToken);

        // Create Gmail service object
        $service = new \Google_Service_Gmail($client);

        // Create a new Gmail message
        $email = new \Google_Service_Gmail_Message();

        // Construct the email content
        $emailContent = [
            'Subject: Meeting',
            'Bcc: ' . implode(',', $bccRecipients), // Combine all Bcc recipients
            '',
            $emailBody // Email body content passed as parameter
        ];
        $emailRaw = implode("\r\n", $emailContent);

        // Set the raw email content to the Gmail message
        $email->setRaw(base64_encode($emailRaw));

        // Send the email using the Gmail API
        $service->users_messages->send("me", $email);

        Log::debug('Email sent successfully.');
    } catch (\Exception $e) {
        Log::error('Error sending email: ' . $e->getMessage());
        throw new \Exception('Failed to send email: ' . $e->getMessage());
    }
}



    public function getGoogleMailerDemailEmails(int $googleUserId)
    {

        $google = GoogleUser::where('google_user_id', $googleUserId)->first();

        $googleAccessToken = $this->checkAccessTokenExpirationAndRefreshIfNeeded($google->google_token, $google->refresh_token,  $google);
        $twentyFourHoursAgo = Carbon::now()->subDays(10);
        $fetchedMessages = Http::withToken($googleAccessToken)->get("https://gmail.googleapis.com/gmail/v1/users/{$google->google_email}/messages", [
            'q' => 'from:mailer-daemon@googlemail.com AND ' . 'after:' . strtotime($twentyFourHoursAgo),
        ]);
        $messageData = $fetchedMessages->json();
        $data = [];
        // Check if there are messages in the response
        if (isset($messageData['messages']) && is_array($messageData['messages'])) {
            // Loop through each message and fetch details
            $messageArray = $messageData['messages'];
            $messageCount =  count($messageArray);
            for ($i = 0; $i < $messageCount; $i++) {
                $messageId = $messageArray[$i]['id'];

                $fetchedMessage = Http::withToken($google->google_token)->withQueryParameters([
                    'maxResults' => 7
                ])->get("https://gmail.googleapis.com/gmail/v1/users/{$google->google_email}/messages/{$messageId}");

                $data[] = $fetchedMessage->json();
            }
        } else {
            Log::debug("No messages found or invalid response format.");
        }
        $failedRecipients = [];
        foreach ($data as $message) {
            if (isset($message['payload']['headers'])) {
                foreach ($message['payload']['headers'] as $header) {
                    if ($header['name'] === 'X-Failed-Recipients') {
                        $failedRecipients[] = $header['value'];
                        // break; // No need to continue searching once found
                    }
                }
            }
        }
        return $failedRecipients;
        // return response()->json([
        //     'failed-emails' => $failedRecipients
        // ]);
    }

    public function getPostMasterEmails(int $googleUserId)
    {
        $google = GoogleUser::where('google_user_id', $googleUserId)->first();
        $googleAccessToken = $this->checkAccessTokenExpirationAndRefreshIfNeeded($google->google_token, $google->refresh_token,  $google);
        $twentyFourHoursAgo = Carbon::now()->subDays(1);

        $fetchedMessages = Http::withToken($googleAccessToken)->get("https://gmail.googleapis.com/gmail/v1/users/{$google->google_email}/messages", [
            'q' => 'from:postmaster@* AND ' . 'after:' . strtotime($twentyFourHoursAgo),
            // 'maxResults' => 1

        ]);

        $messageData = $fetchedMessages->json();
        $data = [];
        // Check if there are messages in the response
        if (isset($messageData['messages']) && is_array($messageData['messages'])) {
            // Loop through each message and fetch details
            $messageArray = $messageData['messages'];
            $messageCount =  count($messageArray);
            for ($i = 0; $i < $messageCount; $i++) {
                $messageId = $messageArray[$i]['id'];
                $fetchedMessage = Http::withToken($google->google_token)->withQueryParameters([
                    'maxResults' => 7
                ])->get("https://gmail.googleapis.com/gmail/v1/users/{$google->google_email}/messages/{$messageId}");

                $data[] = $fetchedMessage->json();
            }
        } else {
            // Handle case where no messages are returned or messages array is not present
            Log::debug("No messages found or invalid response format.");
        }
        $postmasterRecipients = [];
        foreach ($data as $message) {
            if (isset($message['snippet'])) {
                $matches = [];

                // Regular expression to match email addresses
                $pattern = '/\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b/';

                // Perform the regular expression match
                preg_match_all($pattern, $message['snippet'], $matches);

                // Extract the email addresses from the matches
                $emails = $matches[0] ?? [];

                // Output the extracted email addresses
                foreach ($emails as $email) {
                    $postmasterRecipients[] = $email;
                }
            }
        }
        return $postmasterRecipients;
    }


    public function getAutomaticEmails(int $googleUserId)
    {
        $google = GoogleUser::where('google_user_id', $googleUserId)->first();
        $googleAccessToken = $this->checkAccessTokenExpirationAndRefreshIfNeeded($google->google_token, $google->refresh_token,  $google);
        $twentyFourHoursAgo = Carbon::now()->subDays(1);

        $fetchedMessages = Http::withToken($googleAccessToken)->get("https://gmail.googleapis.com/gmail/v1/users/{$google->google_email}/messages", [
            'q' => 'after:' . strtotime($twentyFourHoursAgo) . ' AND subject:(Auto-Reply) OR Auto-Submitted:auto-replied OR Automatic reply',
        ]);
        $messageData = $fetchedMessages->json();
        // return $messageData;
        $data = [];
        // Check if there are messages in the response
        if (isset($messageData['messages']) && is_array($messageData['messages'])) {
            // Loop through each message and fetch details
            $messageArray = $messageData['messages'];
            $messageCount =  count($messageArray);
            for ($i = 0; $i < $messageCount; $i++) {
                $messageId = $messageArray[$i]['id'];
                $fetchedMessage = Http::withToken($google->google_token)->withQueryParameters([
                    'maxResults' => 1
                ])->get("https://gmail.googleapis.com/gmail/v1/users/{$google->google_email}/messages/{$messageId}");

                $data[] = $fetchedMessage->json();
            }
        } else {
            // Handle case where no messages are returned or messages array is not present
            Log::debug("No messages found or invalid response format.");
        }
        $emails = [];
        foreach ($data as $message) {
            if (isset($message['payload']['headers'])) {
                foreach ($message['payload']['headers'] as $header) {
                    if ($header['name'] === 'From') {
                        Log::debug("email " . $header['value']);
                        $matches = [];

                        // Regular expression to match email addresses
                        $pattern = '/<([^>]+)>/';

                        // Perform the regular expression match
                        preg_match($pattern, $header['value'], $matches);

                        // Extract the email address from the matches
                        $email = $matches[1] ?? null;

                        if ($email) {
                            $emails[] = $email;
                        } else {
                            echo "No email found in the string.";
                        }
                    }
                }
            }
        }
        return $emails;
    }

    public function setUpGmailPushNotifications($googleUserId)
    {
        // This function subscribe to the gmail watch to that topic and there is another api which receive the message from that topic
        $googleUser = GoogleUser::where('google_id', $googleUserId)->first();
        $accessToken = $this->checkAccessTokenExpirationAndRefreshIfNeeded($googleUser->google_token, $googleUser->refresh_token, $googleUser);
        $requestData = [
            'topicName' => "projects/salman-370312/topics/mailrecieve-notification",
            'labelIds' => ['UNREAD', 'INBOX'],
            // 'labelFilterAction' => 'include',
            'labelFilterBehavior' => 'include'

        ];
        $response = Http::withToken($accessToken)
            ->post("https://www.googleapis.com/gmail/v1/users/{$googleUser->google_email}/watch", $requestData);

        if ($response->successful()) {
            return response()->json(['success' => true]);
        } else {
            // Handle error
            return response()->json(['error' => 'Failed to set up push notifications'], $response->status());
        }
    }
    public function getGoogleMailerDemailEmailsDataTest(int $googleUserId)
    {

        $google = GoogleUser::where('google_user_id', $googleUserId)->first();

        $twentyFourHoursAgo = Carbon::now()->subDays(10);

        $fetchedMessages = Http::withToken($google->google_token)->get("https://gmail.googleapis.com/gmail/v1/users/{$google->google_email}/messages", [
            //            'q' => 'after:' . strtotime($twentyFourHoursAgo),
            'q' => 'from:mailer-daemon@googlemail.com AND ' . 'after:' . strtotime($twentyFourHoursAgo),

        ]);

        Log::debug([$fetchedMessages]);

        $messageData = $fetchedMessages->json();
        $data = [];

        // Check if there are messages in the response
        if (isset($messageData['messages']) && is_array($messageData['messages'])) {
            // Loop through each message and fetch details
            $messageArray = $messageData['messages'];
            $messageCount =  count($messageArray);
            for ($i = 0; $i < $messageCount; $i++) {
                $messageId = $messageArray[$i]['id'];

                $fetchedMessage = Http::withToken($google->google_token)->withQueryParameters([
                    'maxResults' => 7
                ])->get("https://gmail.googleapis.com/gmail/v1/users/{$google->google_email}/messages/{$messageId}");

                $data[] = $fetchedMessage->json();
            }
        } else {
            // Handle case where no messages are returned or messages array is not present
            Log::debug("No messages found or invalid response format.");
        }
        $failedRecipients = [];
        foreach ($data as $message) {
            if (isset($message['payload']['headers'])) {
                foreach ($message['payload']['headers'] as $header) {
                    if ($header['name'] === 'X-Failed-Recipients') {
                        $failedRecipients[] = $header['value'];
                        break; // No need to continue searching once found
                    }
                }
            }
        }
        // return $failedRecipients;
        return response()->json([
            'failed-emails' => $failedRecipients
        ]);
    }

    function sendMessage($accessToken)
    { // Instantiate the Gmail API client
        Log::debug($accessToken);
        $client = new Google_Client();
        // Load the JSON file contents
        $jsonContents = Storage::disk('local')->get('googleconfig.json');

        // Decode the JSON data
        $credentials = json_decode($jsonContents, true);

        // Use the credentials to configure your client
        $client->setAuthConfig($credentials);

        // Authenticate and authorize
        // Implement your own method to obtain the access token
        $client->setAccessToken($accessToken);
        // Create the Gmail service
        $service = new Google_Service_Gmail($client);
        // Construct the email
        $email = new Google_Service_Gmail_Message();
        $email->setRaw(base64_encode("From: sender@example.com\r\n" . "To: recipient@example.com\r\n" . "Subject: Your subject here\r\n" . "\r\n" . "Email content goes here"));
        // Send the email
        $result = $service->users_messages->send("me", $email);
        if ($result) {
            echo "Email sent successfully!";
        } else {
            echo "Failed to send the email.";
        }
    }

    public function getGooglePubSub(Request $request)
    {
        $data = $request->all();
        PubSubEmailNotification::dispatch($data);
        Log::debug("here");
        return response()->json([
            'success' => 'okay'
        ]);
    }

    public function stopGmailPushNotifications(int $googleUserId)
    {
        $googleUser = GoogleUser::where('google_user_id', $googleUserId)->first();
        $accessToken = $this->checkAccessTokenExpirationAndRefreshIfNeeded($googleUser->google_token, $googleUser->refresh_token, $googleUser);
        $requestData = [
            'topicName' => "projects/salman-370312/topics/mailrecieve-notification",
            'labelIds' => ['INBOX'],
            'labelFilterAction' => 'include'
        ];
        $response = Http::withToken($accessToken)
            ->post("https://gmail.googleapis.com/gmail/v1/users/{$googleUser->google_email}/stop", $requestData);

        if ($response->successful()) {
            return response()->json(['success' => true]);
        } else {
            // Handle error
            return response()->json(['error' => 'Failed to set up push notifications'], $response->status());
        }
    }

    public function getLastHistoryId(string $accessToken)
    {
        $client = new Google_Client();
        $client->setAccessToken($accessToken); // Set the access token obtained during authentication
        $service = new Google_Service_Gmail($client);
        // Call the messages.list method to retrieve a list of messages
        $result = $service->users_messages->listUsersMessages('me');
        Log::debug("messages" , [$result->messages]);

        if(count($result->messages) > 0){
            $message = $result->messages[0];
            Log::debug("message id ", [$message['id']]);

            if(isset($message['id'])){
                $messageId = $message['id'];
                $messageDetails = $service->users_messages->get('me', $messageId);

                Log::debug("Result message ", [$messageDetails]);
                if(isset($messageDetails->historyId)){
                    return $messageDetails->historyId;
                }
                else{
                    return null;
                }
            }


        }
        return null;
    }

    public function demoEmail(Request $request)
    {
        try {
            $userId = $request->user_id;
            $message = $request->message;
            $email = $request->email;
            $googleUser = GoogleUser::where('google_email', $email)->first();

            if (!$googleUser) {
                return response()->json([
                    'message' => 'Google user not found',
                    'success' => false
                ], 404);
            }

            $googleController = new GoogleLoginController();
            $googleAccessToken = $googleController->checkAccessTokenExpirationAndRefreshIfNeeded(
                $googleUser->google_token,
                $googleUser->refresh_token,
                $googleUser
            );

            if (!$googleAccessToken) {
                return response()->json([
                    'message' => 'Failed to obtain Google access token',
                    'success' => false
                ], 500);
            }

            $googleController->sendEmail1($googleAccessToken, ["admin@admin.com", "admin1@admin.com"],$message);

            return response()->json([
                'message' => 'Email sent successfully',
                'success' => true
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
                'success' => false
            ], 500);
        }
    }

}
