<?php

namespace App\Http\Controllers\MailAll;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Google\GoogleLoginController;
use App\Jobs\Gmail\SendEmailJob;
use App\Models\EmailTemplate;
use App\Models\UserLinkedGoogleAccounts;
use App\Services\GmailService;
use App\Services\OAuthService;
use Illuminate\Http\Request;

class EmailMarketingController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        //
        $response =  EmailTemplate::where('user_id',$request->user()->id)->first();
        $data = isset($response) ? $response->toArray() : [];
        return response()->json($data, 200);
    }


    public function sendEmail(Request $request)
    {
        $request->validate([
            'emails' => 'required',
        ]);

        try {
            $emailTemplate =  EmailTemplate::where('user_id',$request->user()->id)->first();
            $userLinkedAccount =  UserLinkedGoogleAccounts::where('user_id',$request->user()->id)->first();

            if(isset($emailTemplate) && isset($userLinkedAccount)){
                $emailTemplateData = isset($emailTemplate) ? $emailTemplate->toArray() : [];
                $userLinkedAccountData = isset($userLinkedAccount) ? $userLinkedAccount->toArray() : [];

                $accessToken = $userLinkedAccountData['access_token'];
                $refreshToken = $userLinkedAccountData['refresh_token'];
                $content = $emailTemplateData['content'];
                $emails = $request->emails;

                $oauthService = new OAuthService();
                $oauthService->setClientId(env('GOOGLE_CLIENT_ID'));
                $oauthService->setClientSecret(env('GOOGLE_CLIENT_SECRET'));
                $oauthService->setRedirectUri(env('GOOGLE_REDIRECT_UNIBOX'));

                $oauthService->setAccessToken($accessToken);
                $oauthService->setRefreshToken($refreshToken);

                $accessToken = $oauthService->checkAccessTokenExpirationAndRefreshIfNeeded();
                SendEmailJob::dispatch($emails, $content, $accessToken);


            }


            return response()->json([
                'message' => 'Email sent successfully',
                'success' => true
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
                'success' => false
            ], 500);
        }
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create(Request $request)
    {
        $request->validate([
            'content' => 'required|string',
        ]);

        // Temp creat or update query just for 1 tempate for the loggged in user if need mutiple tempalte for 1 user remove updateOrCreate and use just create
        $response =  EmailTemplate::updateOrCreate(
            ['user_id' => $request->user()->id], // Attributes to search for
            [
                'content' => $request->content
            ] // Attributes to update or create
        );


        $data = isset($response) ? $response->toArray() : [];
        return response()->json($data, 200);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     */
    public function show(EmailTemplate $emailTemplate)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(EmailTemplate $emailTemplate)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, EmailTemplate $emailTemplate)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(EmailTemplate $emailTemplate)
    {
        //
    }
}
