<?php

namespace App\Http\Controllers\Main;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Traits\PaginateTrait;
use Illuminate\Http\Request;

class ProductController extends Controller
{
    use PaginateTrait;

    public function index(Request $request)
    {
        $input = $request->all();

        $this->setDefaultPaginationValues(input: $input);
        $products = $this->paginatedProductsList();

        return response()->json([
            "products" => $products,
            "status" => true
        ]);
    }

    private function paginatedProductsList()
    {
        $query =  Product::select();
        if (isset($this->searchFilter) && !empty($this->searchFilter)) {
            $query->where('name', 'like', '%' . $this->searchFilter . '%');
        }

        if (isset($this->sortingColumns) && is_array($this->sortingColumns) && count($this->sortingColumns) > 1 && (!empty($this->sortingColumns[0]) && !empty($this->sortingColumns[1]))) {
            $query->orderBy($this->sortingColumns[0], $this->sortingColumns[1]);
        }

        $data = $query->paginate($this->perPage, NULL, NULL, $this->page);

        return isset($data) ? $data->toArray() : [];
    }

    public function allList()
    {
        $data =  Product::get();

        return isset($data) ? $data->toArray() : [];
    }

    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'prefix' => 'required|string|max:255|unique:products',
        ]);

        $product = Product::create($request->all());

        return response()->json([
            "product" => $product,
            "status" => true
        ]);
    }

    public function show(Product $productId)
    {
        return response()->json(["product" => $productId]);
    }

    public function update(Request $request, Product $productId)
    {
        $request->validate([
            'name' => 'sometimes|required|string',
            'prefix' => 'sometimes|required|string|unique:products,prefix,' . $productId->product_id,
        ]);

        $productId->update($request->all());

        return response()->json(["product" => $productId]);
    }

    public function destroy(Product $productId)
    {
        $productId->delete();

        return response()->json(["message" => "Deleted successfully !"], 200);
    }
}
