<?php

namespace App\Http\Controllers\Main;

use App\Http\Controllers\Controller;
use App\Models\ProductPlan;
use App\Traits\PaginateTrait;
use Illuminate\Http\Request;

class ProductPlanController extends Controller
{
    use PaginateTrait;

    public function index(Request $request)
    {
        $input = $request->all();

        $this->setDefaultPaginationValues(input: $input);
        $productPlans = $this->paginatedProductPlansList();

        return response()->json([
            "product_plans"  => $productPlans,
            "status"        => true
        ]);
    }

    private function paginatedProductPlansList()
    {
        $query =  ProductPlan::with('product','children')->select();
        if (isset($this->searchFilter) && !empty($this->searchFilter)) {
            $query->where('title', 'like', '%' . $this->searchFilter . '%');
        }

        if (isset($this->sortingColumns) && is_array($this->sortingColumns) && count($this->sortingColumns) > 1 && (!empty($this->sortingColumns[0]) && !empty($this->sortingColumns[1]))) {
            $query->orderBy($this->sortingColumns[0], $this->sortingColumns[1]);
        }

        $data = $query->paginate($this->perPage, NULL, NULL, $this->page);

        return isset($data) ? $data->toArray() : [];
    }

    public function store(Request $request)
    {
        $request->validate([
            'product_id'        => 'required|exists:products,product_id',
            'parent_id'         => 'nullable|exists:product_plans,product_plan_id',
            'title'             => 'required',
            'no_of_credits'     => 'required',
            'price'             => 'required',
            'validity_number'   => 'required',
            'validity_type'     => 'required',
        ]);

        $productPlan = ProductPlan::create($request->all());

        return response()->json([
            "product_plan"   => $productPlan->load('product'),
            "status"        => true
        ]);
    }

    public function show(ProductPlan $productPlan)
    {
        return response()->json(["product_plan" => $productPlan->load('product')]);

    }

    public function allList()
    {
        $products = ProductPlan::whereNull('parent_id')->with('product')->get();

        return response()->json(['data' => $products]);
    }

    public function getProductPlans(Request $request) {

        $request->validate([
            'productId' => 'required|exists:products,product_id',
        ]);

        $productId = $request->productId;

        $productPlans = ProductPlan::with('product_plan_features')->where('product_id', $productId)->get();

        return response()->json(['data' => $productPlans]);
    }


    public function update(Request $request, ProductPlan $productPlan)
    {
        $request->validate([
            'product_id'    => 'sometimes|required|exists:products,product_id',
            'parent_id'     => 'nullable|exists:product_plans,product_plan_id',
            'title'         => 'sometimes|required',
            'no_of_credits' => 'sometimes|required',
            'price'         => 'sometimes|required',
            'validity'      => 'sometimes|required',
            'validity_type' => 'sometimes|required',
            'is_popular'    => 'sometimes|required',
        ]);

        $productPlan->update($request->all());
        return response()->json(["product_plan" => $productPlan->load('product')]);

    }

    public function destroy(ProductPlan $productPlan)
    {
        $productPlan->delete();
        return response()->json(["message" => "Deleted successfully !"], 200);

    }
}
