<?php

namespace App\Http\Controllers\Main;

use App\Http\Controllers\Controller;
use App\Models\ProductPlanFeature;
use App\Traits\PaginateTrait;
use Illuminate\Http\Request;

class ProductPlanFeatureController extends Controller
{

    use PaginateTrait;

    public function index(Request $request)
    {
        $input = $request->all();

        $this->setDefaultPaginationValues(input: $input);
        $productPlanFeatures = $this->paginatedProductPlanFeaturesList();

        return response()->json([
            "product_plan_features"   => $productPlanFeatures,
            "status"                => true
        ]);
    }

    private function paginatedProductPlanFeaturesList()
    {
        $query =  ProductPlanFeature::with('product_plan')->select();
        if (isset($this->searchFilter) && !empty($this->searchFilter)) {
            $query->where('feature_name', 'like', '%' . $this->searchFilter . '%');
        }

        if (isset($this->sortingColumns) && is_array($this->sortingColumns) && count($this->sortingColumns) > 1 && (!empty($this->sortingColumns[0]) && !empty($this->sortingColumns[1]))) {
            $query->orderBy($this->sortingColumns[0], $this->sortingColumns[1]);
        }

        $data = $query->paginate($this->perPage, NULL, NULL, $this->page);

        return isset($data) ? $data->toArray() : [];
    }

    public function store(Request $request)
    {
        $request->validate([
            'product_plan_id'   => 'required|exists:product_plans,product_plan_id',
            'feature_name'      => 'required',
        ]);

        $productPlanFeature = ProductPlanFeature::create($request->all());

        return response()->json([
            "product_plan_feature" => $productPlanFeature->load('product_plan'),
            "status" => true
        ]);
    }

    public function show(ProductPlanFeature $productPlanFeature)
    {
        return response()->json(["product" => $productPlanFeature]);
    }

    public function update(Request $request, ProductPlanFeature $productPlanFeature)
    {
        $request->validate([
            'product_plan_id'   => 'sometimes|required|exists:product_plans,product_plan_id',
            'feature_name'      => 'sometimes|required',
        ]);

        $productPlanFeature->update($request->all());

        return response()->json(["product_plan_feature" => $productPlanFeature->load('product_plan')]);
    }

    public function destroy(ProductPlanFeature $productPlanFeature)
    {
        $productPlanFeature->delete();
        return response()->json(["message" => "Deleted successfully !"], 200);
    }
}
