<?php

namespace App\Http\Controllers\Main;

use App\Http\Controllers\Controller;
use App\Models\User;
use App\Traits\PaginateTrait;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;

class UserController extends Controller
{
    use PaginateTrait;

    public function index(Request $request)
    {
        $input = $request->all();

        $this->setDefaultPaginationValues(input: $input);
        $users = $this->paginatedUsersList();

        return response()->json([
            "users"   => $users,
            "status"  => true
        ]);
    }

    private function paginatedUsersList()
    {
        $query =  User::select();
        if (isset($this->searchFilter) && !empty($this->searchFilter)) {
            $query->where('name', 'like', '%' . $this->searchFilter . '%');
        }

        if (isset($this->sortingColumns) && is_array($this->sortingColumns) && count($this->sortingColumns) > 1 && (!empty($this->sortingColumns[0]) && !empty($this->sortingColumns[1]))) {
            $query->orderBy($this->sortingColumns[0], $this->sortingColumns[1]);
        }

        $data = $query->paginate($this->perPage, NULL, NULL, $this->page);

        return isset($data) ? $data->toArray() : [];
    }

    public function store(Request $request)
    {
        $data = $request->validate([
            'name'      => 'required|max:255',
            'email'     => 'required|email|unique:users',
            'password'  => 'required',
            'user_type'      => 'integer',
        ]);

        $data['user_type'] = ($request->has('user_type') && $request->user_type != null)
                            ? $request->user_type
                            : 3; // 3 Means Normal user

        $data['password'] = Hash::make($request->password);

        $user = User::create($data);

        return response()->json([
            "user"      => $user,
            "status"    => true
        ]);
    }

    public function show(User $user)
    {
        return response()->json(["user" => $user]);
    }

    public function update(Request $request, User $user)
    {
        $data = $request->validate([
            'name' => 'sometimes|required|max:255',
            'email' => 'sometimes|required|email|unique:users,email,'.$user->id,
            'password' => 'sometimes|required',
            'user_type' => 'sometimes|integer',
        ]);

        if ($request->has('password')) {
            $data['password'] = Hash::make($request->password);
        }

        $user->update($data);

        return response()->json(["user" => $user]);
    }

    public function destroy(User $user)
    {
        $user->delete();
        return response()->json(["message" => "Deleted successfully !"], 200);
    }
}
