<?php

namespace App\Http\Controllers\Main;

use App\Http\Controllers\Controller;
use App\Models\Main\UserSubscription;
use App\Models\Main\UserWallet;
use App\Models\ProductPlan;
use Illuminate\Http\Request;

class UserSubscriptionController extends Controller
{
    public function subscribe(Request $request)
    {
        // Validate incoming request data
        $request->validate([
            'user_id'           => 'required|exists:users,id',
            'product_plan_id'   => 'required|exists:product_plans,product_plan_id',
        ]);

        $productPlan = ProductPlan::where('product_plan_id',$request->product_plan_id)->first();
        $noOfCredits = $productPlan->no_of_credits;

        $wallet = UserWallet::firstOrNew(
            ['user_id' => $request->user_id, 'product_id' => $productPlan->product_id]
        );

        $wallet->credits_balance = $wallet->credits_balance + $noOfCredits;
        $wallet->save();


        $validityTypeMap = [
            'DAY'   => 'days',
            'WEEK'  => 'weeks',
            'MONTH' => 'months',
            'YEAR'  => 'years',
        ];

        // Create new user subscription record
        $subscription = UserSubscription::create([
            'user_id'           => $request->user_id,
            'product_plan_id'   => $request->product_plan_id,
            'start_date'        => now(),
            'end_date'          => now()->add($productPlan->validity_number,  $validityTypeMap[$productPlan->validity_type])
        ]);

        return response()->json(['message' => 'User subscribed successfully.', 'wallet' => $wallet, 'subscription' => $subscription]);
    }

    public function getUserWallet($userId, $productId)
    {
        // Get user's wallet balance for the specified product
        $wallet = UserWallet::where('user_id', $userId)
            ->where('product_id', $productId)
            ->first();

        return response()->json(['wallet' => $wallet]);
    }

    public function getUserSubscriptions(Request $request)
    {

        $userId = $request->user_id;

        $subscriptions = UserSubscription::with("product_plan.product")->where('user_id', $userId)->get();

        return response()->json(['subscriptions' => $subscriptions]);
    }
}
