<?php

namespace App\Http\Controllers\MillionVerify;

use App\Http\Controllers\Controller;
use App\Models\MillionVerifyJob;
use App\Models\VerifiedEmail;
use App\Services\MillionVerify\MillionVerify;
use App\Traits\PaginateTrait;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Config;

class MillionVerifyController extends Controller
{
    use PaginateTrait;

    //
    public function getResults(Request $request)
    {
        $count = isset($request->count) ? $request->count : 10;
        $offset = isset($request->offset) ? $request->offset : 0;


        $apiKey = env('MILLION_VERIFY');
        // config('million.verify');
        $resultFilesDownloader = new MillionVerify($apiKey);

        // Download the list of result files
        $resultFiles = $resultFilesDownloader->downloadResultFiles($count, $offset);

        return response()->json([
            'data' => $resultFiles
        ]);
    }
    public function getEmailTest(Request $request)
    {
        $data = VerifiedEmail::select()->get();

        return response()->json([
            'data' => $data->ToArray()
        ]);
    }

    public function upload(Request $request)
    {
        // Validate the incoming request data
        $request->validate([
            'file' => 'required|mimes:csv,txt|max:10240', // Adjust file types and size as needed
        ]);
        $apiKey = env('MILLION_VERIFY');

        // Store the uploaded file
        $filePath = $request->file('file')->storeAs('uploads', uniqid().'.'.$request->file('file')->extension());
        $resultFilesDownloader = new MillionVerify($apiKey);

        // Specify your MillionVerifier API key
        // Upload the file to MillionVerifier
        $response = null;
        $response = $resultFilesDownloader->uploadFileToMillionVerifier($filePath);
        if(!empty($response)){

            //Log::debug(json_decode($response, true));
            $response = json_decode($response, true);
            if(isset($response['file_id'])){
                MillionVerifyJob::create([
                    'user_id' => $request->user()->id,
                    'file_id' => $response['file_id'],
                    'file_name' => $response['file_name'],

                    'status' => $response['status'],
                    'percent' => $response['percent'],
                    'catch_all' => $response['catch_all'],
                    'is_catch_all_processed' => 0,
                ]);
            }
        }

        // Optionally, you can do further processing with $response

        // Clean up - remove the temporary file
        // Storage::delete($filePath);

        return response()->json(['message' => 'File uploaded successfully', 'response' => $response]);
    }


    public function getPaginatedList(Request $request)
    {
        $fields = [];
        $input = $request->all();
        $this->setDefaultPaginationValues(input: $input);
        $data = $this->pagingatedUserList();

        return response()->json($data);
    }

    public function pagingatedUserList()
    {
        $query =  MillionVerifyJob::select();
        if (isset($this->searchFilter) && !empty($this->searchFilter)) {
            $query->where('name', 'like', '%' . $this->searchFilter . '%')
                ->orWhere('email', 'like', '%' . $this->searchFilter . '%');
        }

        if (isset($this->sortingColumns) && is_array($this->sortingColumns) && count($this->sortingColumns) > 1 && (!empty($this->sortingColumns[0]) && !empty($this->sortingColumns[1]))) {
            $query->orderBy($this->sortingColumns[0], $this->sortingColumns[1]);
        }

        $data = $query->paginate($this->perPage, NULL, NULL, $this->page);

        return isset($data) ? $data->toArray() : [];
    }
    public function getStatusText($statusCode) {
        switch ($statusCode) {
            case 0:
                return 'Pending';
            case 1:
                return 'Legit';
            case 2:
                return 'Bounced';
            case 3:
                return 'Verified';
            case 4:
                return 'Bounced';
            case 5:
                return 'Okay and Automatic Reply';
            default:
                return 'Unknown'; // or handle other cases as needed
        }
    }

    public function downloadCSV(int $jobId)
    {

        $records = VerifiedEmail::where('million_verify_job_id', $jobId)->get(); // Fetch records from the database, replace YourModel with your actual model

        $headers = array(
            "Content-type" => "text/csv",
            "Content-Disposition" => "attachment; filename=records.csv",
            "Pragma" => "no-cache",
            "Cache-Control" => "must-revalidate, post-check=0, pre-check=0",
            "Expires" => "0"
        );
        $callback = function () use ($records) {
            $headerColumn = ["Email","Step 1 Verification Status", "Step 2 Verification Status", "Final Status", "Auto Replies"];

            $file = fopen('php://output', 'w');

            // Add headers to CSV
            fputcsv($file, $headerColumn);

            // Add data to CSV
            foreach ($records as $record) {

                $record = [
                    $record->email,
                    $record->default_status ,
                    $this->getStatusText($record->status) ,
                    $this->getStatusText($record->status),
                    $record->auto_reply_text
                ];

                fputcsv($file, $record);
            }

            fclose($file);
        };

        return response()->stream($callback, 200, $headers);
    }

}
