<?php
// app/Http/Controllers/PlanController.php
namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Plan;
use App\Models\Feature;

class PlanController extends Controller
{
    public function index()
    {
        return Plan::with('features')->get();
    }

    public function store(Request $request)
    {
        $request->validate([
            'name' => 'required|string|unique:plans',
            'price' => 'required|numeric',
            'cost_per_credit' => 'required|numeric',
            'features' => 'required|array',
            'features.*.id' => 'required|exists:features,id',
            'features.*.value' => 'required|string',
        ]);

        $plan = Plan::create($request->only('name', 'price', 'cost_per_credit'));

        foreach ($request->features as $feature) {
            $plan->features()->attach($feature['id'], ['value' => $feature['value']]);
        }

        return response()->json($plan->load('features'), 201);
    }

    public function show($id)
    {
        return Plan::with('features')->findOrFail($id);
    }

    public function update(Request $request, $id)
    {
        $plan = Plan::findOrFail($id);

        $request->validate([
            'name' => 'required|string|unique:plans,name,' . $plan->id,
            'price' => 'required|numeric',
            'cost_per_credit' => 'required|numeric',
            'features' => 'required|array',
            'features.*.id' => 'required|exists:features,id',
            'features.*.value' => 'required|string',
        ]);

        $plan->update($request->only('name', 'price', 'cost_per_credit'));

        $plan->features()->detach();
        foreach ($request->features as $feature) {
            $plan->features()->attach($feature['id'], ['value' => $feature['value']]);
        }

        return response()->json($plan->load('features'), 200);
    }

    public function destroy($id)
    {
        Plan::findOrFail($id)->delete();

        return response()->json(null, 204);
    }
}


?>
