<?php

namespace App\Http\Controllers;

use App\Models\UserLinkedGoogleAccounts;
use Illuminate\Http\Request;
use App\Models\GoogleUser;
use App\Providers\RouteServiceProvider;
use App\Http\Controllers\Controller;
use App\Services\GmailService;
use Google_Client;
use Google_Service_Oauth2;
use Google_Service_Gmail;
use Illuminate\Log\Logger;
use Illuminate\Support\Facades\Log;

class UserLinkedGoogleAccountsController extends Controller
{
    public function redirect()
    {
        $client = new Google_Client();
        $client->setClientId(env('GOOGLE_CLIENT_ID'));
        $client->setClientSecret(env('GOOGLE_CLIENT_SECRET'));
        $client->setRedirectUri(env('GOOGLE_REDIRECT_UNIBOX'));
        $client->setAccessType('offline');
        // $client->setApprovalPrompt('force'); // Add this line

        $client->setScopes(
            [
                'openid',
                'profile',
                'email',
                'https://mail.google.com/',
                'https://www.googleapis.com/auth/gmail.send',
            //    'https://www.googleapis.com/auth/gmail.metadata',
                // 'https://www.googleapis.com/auth/gmail.readonly',
                'https://www.googleapis.com/auth/gmail.modify',
                'https://www.googleapis.com/auth/gmail.labels',
                // 'https://www.googleapis.com/auth/gmail.compose',

                // 'https://www.googleapis.com/auth/userinfo.email',
                // 'https://www.googleapis.com/auth/userinfo.profile',
            ]
        );
        // OLD RETURN
        //  return redirect($client->createAuthUrl());
        // OLD RETURN END
        return $client->createAuthUrl();
    }

    public function callback(Request $request)
    {
        try {
            $client = new Google_Client();
            $client->setClientId(env('GOOGLE_CLIENT_ID'));
            $client->setClientSecret(env('GOOGLE_CLIENT_SECRET'));
            $client->setRedirectUri(env('GOOGLE_REDIRECT_UNIBOX'));
            $client->fetchAccessTokenWithAuthCode($request->get('code'));
            $client->setAccessType('offline');

            // Store or use the access token for later use

            $accessToken = $client->getAccessToken();
            $objOAuthService = new Google_Service_Oauth2($client);
             Log::debug("Handle Google callback ", [$accessToken]);
            if (isset($accessToken)) {
                $userData = $objOAuthService->userinfo->get();

                $email = $userData->getEmail();
                $givenName = $userData->getGivenName();
                $gmailService = new GmailService($accessToken['access_token']);
                //$historyId = 0;
                 $historyId = $gmailService->getLastHistoryId();
                Log::debug("historyId id ", [$historyId]);
                $user = auth()->user();
                $reqUser = request()->user();
                // Use the existing refresh token if not provided in the response
                $refreshToken = null;

                if (isset($accessToken['refresh_token'])) {
                    $refreshToken = $accessToken['refresh_token'];
                }

                // Ensure you handle the case where the refresh token might be missing
                $refreshToken = $refreshToken ?? 'none';


                UserLinkedGoogleAccounts::create([
                    'refresh_token' => $refreshToken,
                    'access_token' => $accessToken['access_token'],
                    'user_id' => $request->user()->id,
                    'last_history_id' => intval($historyId),
                    'email' => $email,
                    'meta_data' => isset($accessToken) ? json_encode($accessToken) : null
                ]);
            }

            // OLD RETURN
            $response = [
                "message" => 'Success', 'google_token' => $accessToken['access_token'],
                'refresh_token' => $accessToken['refresh_token'],
            ];
            return response($response, 200);
        } catch (\Exception $ex) {
            return response(['message' => $ex->getMessage()], 200);
        }
    }
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        //
        $created =  UserLinkedGoogleAccounts::where(
            'user_id',
            $request->user()->id,
        )->first();
        $data = isset($created) ? $created->toArray() : [];
        return response()->json($data, 201);
    }

    public function disconnect(Request $request)
    {
        //
        $created =  UserLinkedGoogleAccounts::where(
            'user_id',
            $request->user()->id,
        )->delete();

        return response()->json(['messsage'=>'Disconnected'], 201);
    }


    /**
     * Show the form for creating a new resource.
     */
    public function create(Request $request)
    {
        //
        $request->validate([
            'refresh_token' => 'required|string',
            'access_token' => 'required|string',
            'email' => 'required|string',
        ]);

        $created =  UserLinkedGoogleAccounts::create([
            'refresh_token' => $request->refresh_token,
            'access_token' => $request->access_token,
            'user_id' => $request->user()->id,
            'email' => $request->email,
            'meta_data' => isset($request->meta_data) ? json_encode($request->meta_data) : null
        ]);
        return response()->json($created, 201);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     */
    public function show(UserLinkedGoogleAccounts $userLinkedGoogleAccounts)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(UserLinkedGoogleAccounts $userLinkedGoogleAccounts)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, UserLinkedGoogleAccounts $userLinkedGoogleAccounts)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(UserLinkedGoogleAccounts $userLinkedGoogleAccounts)
    {
        //
    }
}
