<?php

namespace App\Jobs\Gmail;

use App\Http\Controllers\Google\GoogleLoginController;
use App\Models\GoogleUser;
use App\Models\VerifiedEmail;
use App\Services\OAuthService;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Log\Logger;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Http;

class SyncLinkedUserGmailClientUser implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private $account;

    const BOUNCED = "bounced";
    const AUTO_REPLY = "auto_reply";

    /**
     * Create a new job instance.
     */
    public function __construct($account)
    {
        $this->account = $account;
        //
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {



        if (isset($this->account) && !empty($this->account)) {
            $oauthService = new OAuthService();
            $oauthService->setClientId(env('GOOGLE_CLIENT_ID'));
            $oauthService->setClientSecret(env('GOOGLE_CLIENT_SECRET'));
            $oauthService->setRedirectUri(env('GOOGLE_REDIRECT_UNIBOX'));

            $oauthService->setAccessToken($this->account->access_token);
            $oauthService->setRefreshToken($this->account->refresh_token);

            $accessToken = $oauthService->checkAccessTokenExpirationAndRefreshIfNeeded();
            $requestUrl = "https://gmail.googleapis.com/gmail/v1/users/{$this->account->email}/history";

            // Make the request to fetch the email message
            $response = Http::withToken($accessToken)->withQueryParameters([
                // 'maxResults' => 1,
                'labelId' => 'INBOX',
                'startHistoryId' => $this->account->last_history_id
            ])->get($requestUrl);
            $jsonData = json_decode($response, true);
            $idArray = [];
            if (isset($jsonData['history'])) {
                foreach ($jsonData['history'] as $historyItem) {
                    foreach ($historyItem['messages'] as $message) {
                        // Push id and threadId values into their respective arrays
                        $idArray[] = $message['id'];
                        $threadIdArray[] = $message['threadId'];
                    }
                }
            }
            $this->getMessagesData($idArray, $this->account);
        }
    }



    public function getMessagesData(array $messages, $account)
    {
        $messageCount =  count($messages);
        for ($i = 0; $i < $messageCount; $i++) {
            $messageId = $messages[$i];
            SyncLinkedUserGmailClientUserMessageProcess::dispatch($messageId, $account);
        }
    }
}
