<?php

namespace App\Jobs\Gmail;

use App\Models\UserLinkedGoogleAccounts;
use App\Models\UserLinkedGoogleInbox;
use App\Services\OAuthService;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class SyncLinkedUserGmailClientUserMessageProcess implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private $messageId;
    private $account;

     /**
     * Create a new job instance.
     */
    public function __construct($messageId, $account)
    {
        $this->messageId = $messageId;
        $this->account = $account;
        //
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {
        $oauthService = new OAuthService();
        $oauthService->setClientId(env('GOOGLE_CLIENT_ID'));
        $oauthService->setClientSecret(env('GOOGLE_CLIENT_SECRET'));
        $oauthService->setRedirectUri(env('GOOGLE_REDIRECT_UNIBOX'));

        $oauthService->setAccessToken($this->account->access_token);
        $oauthService->setRefreshToken($this->account->refresh_token);
        $accessToken = $oauthService->checkAccessTokenExpirationAndRefreshIfNeeded();

        $emailData = $this->getMessagesData($this->messageId, $accessToken);

        if (isset($emailData) && is_array($emailData) && count($emailData) > 0) {
            foreach ($emailData as $emailInboxItem) {
                // Log::debug("data ", [$emailInboxItem]);
                if (isset($emailInboxItem['snippet'])) {

                    $fromEmail = null;
                    $action = null;
                    $subject = '';
                    $snippet = $emailInboxItem['snippet'];
                    $historyId = isset($emailInboxItem['historyId']) ? $emailInboxItem['historyId'] : 0;
                    $threadId = isset($emailInboxItem['threadId']) ? $emailInboxItem['threadId'] : '';


                    if (isset($emailInboxItem['payload']['headers'])) {
                        foreach ($emailInboxItem['payload']['headers'] as $header) {
                            if ($header['name'] === 'Subject') {
                                $subject = $header['value'];
                                // Found bounced email
                                break; // No need to continue searching once found
                            }
                        }
                        foreach ($emailInboxItem['payload']['headers'] as $header) {
                            if ($header['name'] === 'Return-Path') {
                                $fromEmail = trim($header['value'], '<>');
                                break;
                            }
                        }
                    }

                    Log::debug("message id ", [$emailInboxItem]);

                    // Update
                    UserLinkedGoogleInbox::create([
                        'user_linked_google_account_id' => $this->account->user_linked_google_account_id,
                        'history_id' => $historyId,
                        'message_id' => $this->messageId,
                        'thread_id' => $threadId,
                        'subject' => $subject,
                        'snippet' => $snippet,
                        'from_email' => $fromEmail

                    ]);

                    if (isset($emailInboxItem['historyId']) && $emailInboxItem['historyId'] !== null) {
                        UserLinkedGoogleAccounts::where('user_linked_google_account_id', $this->account->user_linked_google_account_id)->update(['last_history_id' => $historyId]);
                    }
                }
            }
        }
    }

    public function getMessagesData(string $messageId, string $accessToken)
    {
        $data = [];
        $fetchedMessage = Http::withToken($accessToken)->get("https://gmail.googleapis.com/gmail/v1/users/me/messages/{$messageId}");

        $data[] = $fetchedMessage->json();

        return $data;
    }
}
