<?php

namespace App\Jobs;

use App\Http\Controllers\Google\GoogleLoginController;
use App\Models\GoogleUser;
use App\Models\VerifiedEmail;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

class PoolEmailProcessing implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * Create a new job instance.
     */
    private $millionVerify;

    public function __construct($millionVerify)
    {
        $this->millionVerify = $millionVerify;
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {

        $verifiedEmails = VerifiedEmail::
              where('million_verify_job_id', $this->millionVerify->million_verify_job_id)
            ->where('status', 0)
            ->pluck('email')
            ->toArray();

        $googleUsers = GoogleUser::select()
            ->where('user_id', $this->millionVerify->user_id)
            ->where('mails_sent_in_a_day', '<', 1500)
            ->get();

        $totalEmails = count($verifiedEmails);
        $emailIndex = 0; // Index to track the current email being sent
        $userIndex = 0; // Index to track the current user
        if(isset($googleUsers) && !empty($googleUsers)){
            while ($emailIndex < $totalEmails) {
                // Reset index if all users are iterated
                $googleUser = $googleUsers[$userIndex % count($googleUsers)];

                // Check if the current user has exceeded the limit
                if ($googleUser->mails_sent_in_a_day >= 1500) {
                    // Remove the user from the list
                    unset($googleUsers[$userIndex % count($googleUsers)]);
                    //$googleUsers = array_values($googleUsers); // Reset array keys
                    $googleUsers = $googleUsers->values();

                    continue; // Skip to the next user
                }
                $emailBatch = [];
                // Form email batch
                for ($i = 0; $i < 15 && $emailIndex < $totalEmails; $i++) {
                    $emailBatch[] = $verifiedEmails[$emailIndex];
                    $emailIndex++;
                }
                // Send email batch using the current Google user
                // $googleController = new GoogleLoginController();
                // $googleAccessToken = $googleController->checkAccessTokenExpirationAndRefreshIfNeeded($googleUser->google_token, $googleUser->refresh_token, $googleUser);

                // $googleController->sendEmail($googleAccessToken, $emailBatch);
                echo $googleUser->google_name . "\n";

                ProcessSendEmail::dispatch($googleUser, $emailBatch);

                // Update mails_sent_in_a_day column for the current user by 1 because if we send 15 emails in bcc it counts it as one
                $googleUser->mails_sent_in_a_day += 15;
                $googleUser->save();

                $userIndex++; // Move to the next user
            }
        }

    }
}
