<?php

namespace App\Jobs;

use App\Http\Controllers\Google\GoogleLoginController;
use App\Models\GoogleUser;
use App\Models\VerifiedEmail;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Log\Logger;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Http;

class PubSubEmailNotification implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private $data;
    private $googleUser;

    const BOUNCED = "bounced";
    const AUTO_REPLY = "auto_reply";

    /**
     * Create a new job instance.
     */
    public function __construct($data)
    {
        $this->data = $data;
        //
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {
        //
        //Log::debug("this--> ", [$this->data]);
        $messageData = $this->data['message']['data'];  // 'eyJlbWFpbEFkZHJlc3MiOiJhbGlAbWFpbGFsbC5pbyIsImhpc3RvcnlJZCI6MjIwNzR9';
        // Decode the base64-encoded data
        $messagePayload = base64_decode($messageData);

        // Convert the JSON payload to an associative array
        $messageArray = json_decode($messagePayload, true);
        $sentFromEmail =  $messageArray['emailAddress'];
        $historyId =  $messageArray['historyId'];

        // Log::debug("this--> ", [$messageArray]);

        $googleUser = GoogleUser::where('google_email', $sentFromEmail)->first();
        if (isset($googleUser) && !empty($googleUser)) {
            $googleControler = new GoogleLoginController();
            $accessToken = $googleControler->checkAccessTokenExpirationAndRefreshIfNeeded($googleUser->google_token, $googleUser->refresh_token, $googleUser);

            // Construct the request URL to fetch the email message
            $requestUrl = "https://gmail.googleapis.com/gmail/v1/users/{$sentFromEmail}/history";

            // Make the request to fetch the email message
            $response = Http::withToken($accessToken)->withQueryParameters([
                // 'maxResults' => 1,
                'startHistoryId' => $historyId
            ])->get($requestUrl);
            $jsonData = json_decode($response, true);
            $idArray = [];
            // Log::debug("repose ", [$jsonData]);

            if (isset($jsonData['history'])) {
                foreach ($jsonData['history'] as $historyItem) {
                    foreach ($historyItem['messages'] as $message) {
                        // Push id and threadId values into their respective arrays
                        $idArray[] = $message['id'];
                        $threadIdArray[] = $message['threadId'];
                    }
                }
            }

            Log::debug("idArray ", [$idArray]);
            $this->getMessagesData($idArray, $googleUser);

            // if (isset($emailData) && is_array($emailData) && count($emailData) > 0) {
            //     foreach ($emailData as $emailInboxItem) {
            //         // Log::debug("data ", [$emailInboxItem]);
            //         if(isset($emailInboxItem['snippet'])){

            //             $foundEmail = null;
            //             $action = null;
            //             $subject = '';
            //             // Check header or X-failed-Recipients
            //             if (isset($emailInboxItem['payload']['headers'])) {
            //                 foreach ($emailInboxItem['payload']['headers'] as $header) {
            //                     if ($header['name'] === 'X-Failed-Recipients') {
            //                         $failedRecipients[] = $header['value'];
            //                         $foundEmail = $header['value'];
            //                         $action = self::BOUNCED;
            //                         // Found bounced email
            //                         break; // No need to continue searching once found
            //                     }
            //                 }
            //             }

            //             if (isset($emailInboxItem['payload']['headers'])) {
            //                 foreach ($emailInboxItem['payload']['headers'] as $header) {
            //                     if ($header['name'] === 'Subject') {
            //                         $subject = $header['value'];

            //                         // Found bounced email
            //                         break; // No need to continue searching once found
            //                     }
            //                 }
            //             }

            //             if ($this->detectPostMasterEmail($emailInboxItem['snippet'], $subject)) {

            //                 if ($foundEmail === null && $action === null) {
            //                     if (isset($emailInboxItem['snippet'])) {
            //                         $matches = [];

            //                         // Regular expression to match email addresses
            //                         $pattern = '/\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Z|a-z]{2,}\b/';

            //                         // Perform the regular expression match
            //                         preg_match_all($pattern, $emailInboxItem['snippet'], $matches);

            //                         // Extract the email addresses from the matches
            //                         $emails = $matches[0] ?? [];

            //                         // Output the extracted email addresses
            //                         foreach ($emails as $email) {
            //                             $foundEmail =  $email;
            //                             $action = self::BOUNCED;
            //                         }
            //                     }
            //                 }
            //             }

            //             if ($foundEmail === null && $action === null) {

            //                 if ($this->detectAutomaticReply($emailInboxItem['snippet'], $subject)) {
            //                     if (isset($emailInboxItem['payload']['headers'])) {
            //                         foreach ($emailInboxItem['payload']['headers'] as $header) {
            //                             if ($header['name'] === 'From') {
            //                                 $matches = [];

            //                                 // Regular expression to match email addresses
            //                                 $pattern = '/<([^>]+)>/';

            //                                 // Perform the regular expression match
            //                                 preg_match($pattern, $header['value'], $matches);

            //                                 // Extract the email address from the matches
            //                                 $email = $matches[1] ?? null;

            //                                 if ($email) {
            //                                     $foundEmail =  $email;
            //                                     $action = self::AUTO_REPLY;
            //                                     // add autoreply email
            //                                 }
            //                             }
            //                         }
            //                     }
            //                 }
            //             }

            //             Log::debug($action);
            //             Log::debug($foundEmail);
            //             $foundEmail = strtolower($foundEmail);


            //             $verified = VerifiedEmail::where('email',$foundEmail)->first();
            //             if(isset($verified) && !empty($verified) && $foundEmail !== null && $action !== null ){
            //                 Log::debug("Found Email for saving ", [$verified]);
            //                 $verified->result = "COMPLETED";
            //                 if($action === self::BOUNCED){
            //                     Log::debug("Save Bounce status ", [$verified]);
            //                     $verified->status = 4;


            //                 }
            //                 if($action === self::AUTO_REPLY){
            //                     $verified->status = 5 ; // Auto Reply
            //                     $verified->email_inbox_checking_status = 'AUTO_REPLY_FOUND';
            //                 }
            //                 $verified->save();
            //             }


            //             # code...
            //         }

            //     }
            // }
        }
    }



    public function getMessagesData(array $messages, $googleUser)
    {
        $data = [];
        $messageCount =  count($messages);
        for ($i = 0; $i < $messageCount; $i++) {
            $messageId = $messages[$i];
            PubSubEmailNotificationProcess::dispatch($messageId, $googleUser);
            // $fetchedMessage = Http::withToken($accessToken)->withQueryParameters([
            //     'maxResults' => 1
            // ])->get("https://gmail.googleapis.com/gmail/v1/users/me/messages/{$messageId}");

            // $data[] = $fetchedMessage->json();
        }

        // return $data;
    }

    // public function detectEmail($emailData)
    // {
    // }


    // // Function to detect automatic replies
    // function detectAutomaticReply($subject, $body)
    // {
    //     // List of common phrases indicating automatic replies
    //     $automaticReplyPhrases = [
    //         "Out of Office",
    //         "Auto-reply",
    //         "Automated Response",
    //         "Automatic reply",
    //         "Taking a short break",
    //         "Currently out of office",
    //         "Enjoying some time off",
    //         "Away for a while",
    //         "Temporarily offline",
    //         "On a quick vacation",
    //         "Out and about",
    //         "Away from my desk",
    //         "Taking a breather",
    //         "Out and about, back soon",
    //         "Out of the office break",
    //         "Currently offline, back shortly",
    //         "I am out of office"
    //         // Add more phrases as needed
    //     ];

    //     // Search for automatic reply phrases in the subject and body
    //     foreach ($automaticReplyPhrases as $phrase) {
    //         if (stripos($subject, $phrase) !== false || stripos($body, $phrase) !== false) {
    //             return true;
    //         }
    //     }
    //     return false;
    // }

    // // Function to detect bounced emails
    // function detectBouncedEmail($subject, $body)
    // {
    //     // List of common phrases indicating bounce notifications
    //     $bouncePhrases = [
    //         "Delivery Status Notification",
    //         "Undelivered Mail Returned to Sender",
    //         // Add more phrases as needed
    //     ];

    //     // Search for bounce notification phrases in the subject and body
    //     foreach ($bouncePhrases as $phrase) {
    //         if (stripos($subject, $phrase) !== false || stripos($body, $phrase) !== false) {
    //             return true;
    //         }
    //     }
    //     return false;
    // }


    // function detectPostMasterEmail($subject,$body)
    // {
    //     $bouncePhrases = [
    //         "Delivery Status Notification",
    //         "Undelivered Mail Returned to Sender",
    //         "postmaster",
    //         "couldn&#39;t be delivered",
    //         "wasn&#39;t found at",
    //         "wasn't found at",
    //         "couldn't be delivered",
    //         "Delivery has failed to these recipients",
    //         "was undeliverable",
    //         "rejected your message to the following email addresses"

    //         // Add more phrases as needed
    //     ];
    //     Log::debug($subject);
    //     $subject = html_entity_decode($subject);

    //     // Search for bounce notification phrases in the subject and body
    //     foreach ($bouncePhrases as $phrase) {
    //         if (stripos($subject, $phrase) !== false || stripos($body, $phrase) !== false) {
    //             Log::debug("Postmater bounced detected");
    //             return true;
    //         }
    //     }
    //     Log::debug("Postmater not bounced detected");

    //     return false;


    // }
}
