<?php

namespace App\Jobs;

use App\Http\Controllers\Google\GoogleLoginController;
use App\Models\SyncMailResults;
use App\Models\VerifiedEmail;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Log\Logger;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

class StartEmailCheckFromGoogleAPI implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private $verifiedEmails;
    private $google_user_id;
    /**
     * Create a new job instance.
     */
    public function __construct($verifiedEmails, $googleUserId)
    {
        $this->verifiedEmails = $verifiedEmails;
        $this->google_user_id = $googleUserId;
        //
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {
        //
        $googleControler = new GoogleLoginController();
        $bouncedEmailArray =     SyncMailResults::where('google_user_id', $this->google_user_id)->where('result',SyncMailResults::BOUNCED )->pluck('result_email')->toArray();
        $autoReplyEmails =     SyncMailResults::where('google_user_id', $this->google_user_id)->where('result',SyncMailResults::AUTO_REPLY)->pluck('result_email')->toArray();

        // $mailsFromMailerDaemon = $googleControler->getGoogleMailerDemailEmails($this->google_user_id);
        // $mailsFromPostMaster = $googleControler->getPostMasterEmails($this->google_user_id);
        // $autoReplyEmails = $googleControler->getAutomaticEmails($this->google_user_id);

        //$bouncedEmailArray = array_merge($mailsFromMailerDaemon,$mailsFromPostMaster);
        //Log::debug("this emails not found  === " .count($bouncedEmailArray));

        foreach ($this->verifiedEmails as $verifiedEmail) {
            $verified = VerifiedEmail::where('id',$verifiedEmail->id)->first();
            if (in_array($verifiedEmail->email, $bouncedEmailArray)) {
                $verified->status = VerifiedEmail::INVALID_STATUS;
                $verified->result = VerifiedEmail::RESULT_COMPLETED;
            } else {
                $auto_reply_status = "";
                $verified->status = VerifiedEmail::VALID_STATUS;
                $auto_reply_status  = 'AUTO_REPLY_NOT_FOUND';

                if (in_array($verifiedEmail->email, $autoReplyEmails)) {
                    $auto_reply_status  = 'AUTO_REPLY_FOUND';
                    $verified->result = VerifiedEmail::RESULT_COMPLETED;
                    $autoReplyResult = SyncMailResults::where('result_email', $verifiedEmail->email)->first();
                    if(isset($autoReplyResult) && !empty($autoReplyResult)){
                        $verified->auto_reply_text = $autoReplyResult->result_text;

                    }


                }
                $verified->email_inbox_checking_status = $auto_reply_status;

            }
            $verified->result = VerifiedEmail::RESULT_COMPLETED;


            $verified->save();
        }

    }
}
