<?php

namespace App\Jobs;

use App\Models\GoogleUser;
use App\Models\MillionVerifyJob;
use App\Models\VerifiedEmail;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

class StartEmailProcess implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * Create a new job instance.
     */
    private $data;
    private $filename;
    private $millionVerify;
    private $emailColumnIndex;

    public function __construct($filename, $millionVerify)
    {
        //
        // $this->data = $csvData;
        $this->filename = $filename;
        $this->millionVerify = $millionVerify;
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {

        $path = 'app/uploads/catchall/' . $this->filename;
        $csvFile = fopen(storage_path($path), 'r');
        if ($csvFile === false) {
            die("Error opening file: $path");
        }
        $header = fgetcsv($csvFile);
        if ($header === false) {
            die("Error reading header row");
        }
        $emailColumnIndex = null;
        $resultColumnIndex = null;
        $firstRow = fgetcsv($csvFile);
        if ($firstRow !== false && !empty($firstRow)) {
            foreach ($firstRow as $index => $value) {
                if (filter_var($value, FILTER_VALIDATE_EMAIL)) {
                    $emailColumnIndex = $index;
                    break;

                }
                if (strcasecmp($value, 'result') === 0) {
                    $resultColumnIndex = $index;
                }
                if ($emailColumnIndex !== null && $resultColumnIndex !== null) {
                    break;
                }
            }
        }
        if ($header !== false && !empty($header)) {
            foreach ($header as $index => $value) {
                if (strcasecmp($value, 'result') === 0) {
                    $resultColumnIndex = $index;
                    break;
                }
            }
        }

        if ($emailColumnIndex >= 0) { // When Email column Index found
            $this->emailColumnIndex = $emailColumnIndex;
            $emails = [
                'invalid' => [],
                'ok' => [],
                'unknown' => [],
                'catch_all' => [],
            ];

            rewind($csvFile);
            // Skip the header row
            fgetcsv($csvFile);
            // Read the remaining rows of the CSV file
            while (($row = fgetcsv($csvFile)) !== false) {
                // Skip rows that are empty or contain only whitespace characters
                if (array_filter($row)) {
                    // Extract email from the row
                    $email = $row[$emailColumnIndex];

                    // Add email to the array
                    $quality = $row[$resultColumnIndex];

                    // Add email to the appropriate quality array
                    switch ($quality) {
                        case 'invalid':
                            $emails['invalid'][] = $email;
                            break;
                        case 'ok':
                            $emails['ok'][] = $email;
                            break;
                        case 'unknown':
                            $emails['unknown'][] = $email;
                            break;
                        case 'catch_all':
                            $emails['catch_all'][] = $email;
                            break;
                        default:
                            // Handle unknown quality (if necessary)
                            break;
                    }

                }
            }

            // STORED IN DATABASE
            $this->storeEmails($emails,$this->millionVerify->million_verify_job_id);
            // All the emails from million verify has been taken into an array and then process for the email sending
            MillionVerifyJob::where('is_catch_all_processed', $this->millionVerify->million_verify_job_id)->update(['is_catch_all_processed' => MillionVerifyJob::IS_CATCH_ALL_PROCESSED_PROCESSED]);
            // Close the file handle
            fclose($csvFile);
           // PoolEmailProcessing::dispatch($this->millionVerify); // Commented this because we are making cron command for big files

        }

    }


    private function storeEmails($emails, $millionVerifyJobId)
    {
        if (count($emails['invalid'])) {
            foreach ($emails['invalid'] as $invalidEmail) {
                if (!VerifiedEmail::where('email', $invalidEmail)->where('million_verify_job_id',$millionVerifyJobId)->exists()) {
                    VerifiedEmail::create([
                        'default_status'=> VerifiedEmail::DEFAULT_STATUS_INVALID,
                        'email_column_index' => $this->emailColumnIndex,
                        'email' => $invalidEmail,
                        'google_user_id' => 0,
                        'is_sent'=> VerifiedEmail::SENT, // Dont send and check the invalid and okay emails
                        'status' => VerifiedEmail::INVALID_STATUS,
                        'million_verify_job_id' => $millionVerifyJobId
                    ]);
                }
            }
        }

        if (count($emails['ok'])) {
            foreach ($emails['ok'] as $okEmail) {
                if (!VerifiedEmail::where('email', $okEmail)->where('million_verify_job_id',$millionVerifyJobId)->exists()) {
                    VerifiedEmail::create([
                        'default_status'=> VerifiedEmail::DEFAULT_STATUS_OK,
                        'email_column_index' => $this->emailColumnIndex,
                        'email' => $okEmail,
                        'google_user_id' => 0,
                        'is_sent'=> VerifiedEmail::SENT, // Dont send and check the invalid and okay emails
                        'status' => VerifiedEmail::VALID_STATUS,
                        'million_verify_job_id' => $millionVerifyJobId
                    ]);
                }
            }
        }

       // $all = array_merge($emails['unknown'],$emails['catch_all']);
        foreach ($emails['unknown'] as $other) {
            if (!VerifiedEmail::where('email', $other)->where('million_verify_job_id',$millionVerifyJobId)->exists()) {

                VerifiedEmail::create([
                    'default_status'=> VerifiedEmail::DEFAULT_STATUS_UNKNOWN,
                    'email_column_index' => $this->emailColumnIndex,
                    'email' => $other,
                    'google_user_id' => 0,
                    'status' => VerifiedEmail::PENDING_STATUS, // PEnding because have not check inbox
                    'is_sent'=> VerifiedEmail::NOT_SENT, // Not Sent Email Will process in Pool Email process cron because this is 0
                    'million_verify_job_id' => $millionVerifyJobId
                ]);
            }
        }
        foreach ($emails['catch_all'] as $other) {
            if (!VerifiedEmail::where('email', $other)->where('million_verify_job_id',$millionVerifyJobId)->exists()) {

                VerifiedEmail::create([
                    'default_status'=> VerifiedEmail::DEFAULT_STATUS_CATCH_ALL,
                    'email_column_index' => $this->emailColumnIndex,
                    'email' => $other,
                    'google_user_id' => 0,
                    'status' => VerifiedEmail::PENDING_STATUS, // PEnding because have not check inbox
                    'is_sent'=> VerifiedEmail::NOT_SENT, // Not Sent Email Will process in Pool Email process cron because this is 0
                    'million_verify_job_id' => $millionVerifyJobId
                ]);
            }
        }
        // if (count($all)) {

        //     foreach ($all as $other) {
        //         if (!VerifiedEmail::where('email', $other)->exists()) {

        //             VerifiedEmail::create([
        //                 'email' => $other,
        //                 'google_user_id' => 0,
        //                 'status' => VerifiedEmail::PENDING_STATUS, // PEnding because have not check inbox
        //                 'is_sent'=> VerifiedEmail::NOT_SENT, // Not Sent Email Will process in Pool Email process cron because this is 0
        //                 'million_verify_job_id' => $millionVerifyJobId
        //             ]);
        //         }
        //     }
        // }

    }


}
