<?php

namespace App\Services;

use Google_Client;
use Illuminate\Support\Facades\Log;
use Google_Service;
use Google_Service_Gmail;
use Google_Service_Oauth2;
use Google_Service_Gmail_Message;
use Google_Service_Gmail_MessagePart;
use Google_Service_Gmail_MessagePayload;
use Google_Service_Gmail_MessagePartHeader;
use Google\Service\Gmail;
use Google\Service\Gmail\Message;
use Google\Service\Gmail\MessagePartHeader;

class GmailService {

    private $accessToken;


    public function __construct(string $accessToken = "") {
        $this->accessToken = $accessToken;
    }


    public function sendEmail($bccRecipients, $emailBody)
    {
        try {
            // Instantiate Google Client
            $client = new Google_Client();
            $client->setAccessToken($this->accessToken);

            // Create Gmail service object
            $service = new \Google_Service_Gmail($client);

            // Create a new Gmail message
            $email = new \Google_Service_Gmail_Message();

            // Construct the email content
            $emailContent = [
                'Subject: Meeting',
                'Bcc: ' . implode(',', $bccRecipients), // Combine all Bcc recipients
                '',
                $emailBody // Email body content passed as parameter
            ];
            $emailRaw = implode("\r\n", $emailContent);

            // Set the raw email content to the Gmail message
            $email->setRaw(base64_encode($emailRaw));

            // Send the email using the Gmail API
            $service->users_messages->send("me", $email);

            Log::debug('Email sent successfully.');
        } catch (\Exception $e) {
            Log::error('Error sending email: ' . $e->getMessage());
            throw new \Exception('Failed to send email: ' . $e->getMessage());
        }
    }

    public function getLastHistoryId()
    {
        $client = new Google_Client();
        $client->setAccessToken($this->accessToken); // Set the access token obtained during authentication
        $service = new Google_Service_Gmail($client);
        // Call the messages.list method to retrieve a list of messages
        $result = $service->users_messages->listUsersMessages('me');

        if(count($result->messages) > 0){
            $message = $result->messages[0];
            Log::debug("message id ", [$message['id']]);

            if(isset($message['id'])){
                $messageId = $message['id'];
                $messageDetails = $service->users_messages->get('me', $messageId);

                Log::debug("Result message ", [$messageDetails]);
                if(isset($messageDetails->historyId)){
                    return $messageDetails->historyId;
                }
                else{
                    return null;
                }
            }
        }
        return null;
    }
}
