<?php

namespace App\Services\MillionVerify;

use Illuminate\Log\Logger;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class MillionVerify
{
    private $apiKey;

    public function __construct($apiKey)
    {
        $this->apiKey = $apiKey;
    }

    /**
     * Check the status of a file verification process.
     *
     * @param string $fileId
     * @return string|null Status of the verification process or null on failure
     */
    public function checkFileStatus($fileId)
    {
        $url = "https://bulkapi.millionverifier.com/bulkapi/v2/fileinfo?key={$this->apiKey}&file_id={$fileId}";

        // Make a cURL request to the API
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        $response = curl_exec($ch);

        // Check if cURL request was successful
        if (curl_errno($ch)) {
            // Handle error
            curl_close($ch);
            return null;
        }

        curl_close($ch);

        // Parse the JSON response
        $data = json_decode($response, true);

        // Check if the response contains the 'status' key
        return $data;
    }

    public function downloadCSV($fileId, $filter = '')
    {
        // Build the API download URL
        $url = "https://bulkapi.millionverifier.com/bulkapi/v2/download?key={$this->apiKey}&file_id={$fileId}&filter=all";

        // Add filter if provided
        if (!empty($filter)) {
            // $url .= "&filter=custom&statuses=catch_all,unknown";
           // $url .= "&filter=all";

        }
        Log::debug($url);
        // Download the CSV file
        $csvContent = file_get_contents($url);

        // Check if the download was successful
        if ($csvContent === false) {
            // Handle error
            return false;
        }

        return $csvContent;
    }


    public function downloadResultFiles($count = 10, $offset = 0)
    {
        // Build the API endpoint URL
        $url = "https://app.millionverifier.com/bulk-api/result-files?key={$this->apiKey}&count={$count}&offset={$offset}";

        // Download the result files list
        $result = file_get_contents($url);

        // Check if the download was successful
        if ($result === false) {
            // Handle error
            return false;
        }

        // Parse the JSON response
        $resultFiles = json_decode($result, true);

        // Check if the response contains the 'files' key
        if (isset($resultFiles['results'])) {
            return $resultFiles;
        }

        return false;
    }

    function uploadFileToMillionVerifier($filePath)
    {
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => "https://bulkapi.millionverifier.com/bulkapi/v2/upload?key={$this->apiKey}",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => array('file_contents' => new \CURLFILE(storage_path("/app/" . $filePath))),
        ));

        $response = curl_exec($curl);

        curl_close($curl);


        return $response;
    }
}
