<?php
namespace App\Services\MillionVerify;

use App\Models\User;
use App\Models\Plan;

class SubscriptionService
{
    public function createSubscription(User $user, Plan $plan)
    {
        return $user->subscriptions()->create([
            'plan_id' => $plan->id,
            'starts_at' => now(),
            'ends_at' => now()->addMonth(),
        ]);
    }

    public function trackUsage(User $user)
    {
        $today = now()->toDateString();
        $usage = $user->usage()->firstOrCreate(
            ['date' => $today],
            ['requests' => 0]
        );

        $plan = $user->subscriptions()->latest()->first()->plan;
        $limit = $plan->features()->where('name', 'Validation Credits')->first()->pivot->value;

        if ($usage->requests >= $limit) {
            throw new \Exception('Request limit reached');
        }

        $usage->increment('requests');
    }
}

?>
