<?php

use App\Http\Controllers\Auth\UserAuthController;
use App\Http\Controllers\CloudLead\CustomLeadRequestController;
use App\Http\Controllers\CloudLead\CustomLeadRequestResponseController;
use App\Http\Controllers\CloudLead\EnrichmentController;
use App\Http\Controllers\CloudLead\EnrichmentResponseController;
use App\Http\Controllers\EmailTemplateController;
use App\Http\Controllers\ExternalLoginController;
use App\Http\Controllers\Google\GoogleUserController;
use App\Http\Controllers\Google\GoogleLoginController;
use App\Http\Controllers\MailAll\EmailMarketingController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Mail;
use Illuminate\Validation\ValidationException;
// use App\Http\Controllers\Google\UserController;
use App\Http\Controllers\Main\ProductController;
use App\Http\Controllers\Main\ProductPlanController;
use App\Http\Controllers\Main\ProductPlanFeatureController;
use App\Http\Controllers\Main\UserController;
use App\Http\Controllers\Main\UserSubscriptionController;
use App\Http\Controllers\MillionVerify\MillionVerifyController;
use App\Http\Controllers\PlanController;
use App\Http\Controllers\SubscriptionController;
use App\Http\Controllers\UserLinkedGoogleAccountsController;
use App\Http\Controllers\UserLinkedGoogleInboxController;
use App\Jobs\StartEmailProcess;
use App\Models\EmailTemplate;
use App\Models\GoogleUser;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider and all of them will
| be assigned to the "api" middleware group. Make something great!
|
*/

// Route::middleware('auth:sanctum')->get('/user', function (Request $request) {
//     return $request->user();
// });

Route::middleware('auth:api')->group(function () {

    Route::get('/me', [UserAuthController::class, 'me']);
    Route::get('/fetch/users', [GoogleUserController::class, 'getUsers']);

    Route::group(['prefix' => 'user-google-linked-account'], function () {
        Route::get('/redirect', [UserLinkedGoogleAccountsController::class, 'redirect']);
        Route::get('/callback', [UserLinkedGoogleAccountsController::class, 'callback']);
        Route::post('/', [UserLinkedGoogleAccountsController::class, 'create']);
        Route::get('/', [UserLinkedGoogleAccountsController::class, 'index']);
        Route::get('/disconnect', [UserLinkedGoogleAccountsController::class, 'disconnect']);

    });

    Route::group(['prefix' => 'user-google-account-inbox'], function () {
        Route::post('/', [UserLinkedGoogleInboxController::class, 'create']);
        Route::get('/list/paginated', [UserLinkedGoogleInboxController::class, 'index']);
    });

    Route::group(['prefix' => 'email-template'], function () {
        Route::post('/', [EmailTemplateController::class, 'create']);
        Route::get('/{userId}', [EmailTemplateController::class, 'index']);
    });


    Route::group(['prefix' => 'million-verify'], function () {
        Route::get('/', [MillionVerifyController::class, 'getResults']);
        Route::post('/upload', [MillionVerifyController::class, 'upload']);
        Route::get('/paginated', [MillionVerifyController::class, 'getPaginatedList']);
        Route::get('/{jobId}/download', [MillionVerifyController::class, 'downloadCSV']);
        Route::get('/get/emails', [MillionVerifyController::class, 'getEmailTest']);
    });

    Route::post('/subscribe', [SubscriptionController::class, 'subscribe']);
    // Route::middleware('check.usage')->get('/resource', [ResourceController::class, 'access']); // Ye hum jb use karain jb file upload me



    // NEW ROUTES FOR MAIN APP
    // PRODUCTS
    Route::get('product/list/paginated', [ProductController::class, 'index']);
    Route::post('product', [ProductController::class, 'store']);
    Route::get('product/{productId}', [ProductController::class, 'show']);
    Route::put('product/{productId}', [ProductController::class, 'update']);
    Route::get('product/all', [ProductController::class, 'allList']);
    Route::delete('product/{productId}', [ProductController::class, 'destroy']);


    // PRODUCT PLAN FEATURES
    // Route::get('product-plan-features', [ProductPlanFeatureController::class, 'index']);
    // Route::post('product-plan-features', [ProductPlanFeatureController::class, 'store']);
    // Route::get('product-plan-features/{productPlanFeature}', [ProductPlanFeatureController::class, 'show']);
    // Route::put('product-plan-features/{productPlanFeature}', [ProductPlanFeatureController::class, 'update']);
    // Route::delete('product-plan-features/{productPlanFeature}', [ProductPlanFeatureController::class, 'destroy']);
});



Route::get('/', function () {
    return "API WORKING";
});

Route::post('/signup', [UserAuthController::class, 'register']);
Route::post('/signin', [UserAuthController::class, 'login']);
Route::post('/checkemail', [UserAuthController::class, 'checkemail']);
Route::post('/refresh-token', [UserAuthController::class, 'refreshToken']);


// Route::get('/google/redirect', [GoogleLoginController::class, 'redirectToGoogle'])->name('google.redirect'); // This is in working condition so thats why comment and testing into new functions
// Route::get('/google/callback', [GoogleLoginController::class, 'handleGoogleCallback'])->name('google.callback');

Route::get('/google/redirect', [GoogleLoginController::class, 'redirectToGoogleNew']);
Route::get('/google/callback', [GoogleLoginController::class, 'handleGoogleCallback']);

Route::group(['prefix' => 'email-marketing'], function () {
    Route::post("/send/email",[EmailMarketingController::class,"sendEmail"]);

});


Route::get('product/all/fetch', [ProductController::class, 'allList']);
Route::get('product-plan/all/fetch', [ProductPlanController::class, 'allList']);
// PRODUCT PLANS
Route::get('product-plans/list/paginated', [ProductPlanController::class, 'index']);
Route::post('product-plans', [ProductPlanController::class, 'store']);
Route::get('product-plans/{productPlan}', [ProductPlanController::class, 'show']);
Route::put('product-plans/{productPlan}', [ProductPlanController::class, 'update']);
Route::delete('product-plans/{productPlan}', [ProductPlanController::class, 'destroy']);

Route::get('product-plans/product/list', [ProductPlanController::class, 'getProductPlans']);

Route::group(['prefix' => 'subscription'], function () {
    Route::get('/list/paginated', [ProductPlanFeatureController::class, 'index']);
    Route::post('/', [ProductPlanFeatureController::class, 'store']);
    Route::get('/{productPlanFeature}', [ProductPlanFeatureController::class, 'show']);
    Route::put('/{productPlanFeature}', [ProductPlanFeatureController::class, 'update']);
    Route::delete('/{productPlanFeature}', [ProductPlanFeatureController::class, 'destroy']);
});

Route::group(['prefix' => 'users'], function () {
    Route::get('/list/paginated', [UserController::class, 'index']);
    Route::post('/', [UserController::class, 'store']);
    Route::get('/{user}', [UserController::class, 'show']);
    Route::put('/{user}', [UserController::class, 'update']);
    Route::delete('/{user}', [UserController::class, 'destroy']);
});




Route::group(['prefix' => 'subscription'], function () {
    Route::post('/product/user', [UserSubscriptionController::class, 'subscribe']);
    Route::get('/fetch-all-user-subscriptions', [UserSubscriptionController::class, 'getUserSubscriptions']);
});


// Route::get('/get-inbox/{googleUserId}',[GoogleLoginController::class, 'getGoogleMailerDemailEmailsDataTest']);
// Route::get('/get-postmaster/{googleUserId}',[GoogleLoginController::class, 'getPostMasterEmails']);
// Route::get('/get-auto/{googleUserId}',[GoogleLoginController::class, 'getAutomaticEmails']);

// Route::get('/get-pubsub',[GoogleLoginController::class, 'getGooglePubSub']);
// Route::post('/get-pubsub',[GoogleLoginController::class, 'getGooglePubSub']);
//Route::get('/stop-pubsub/{googleUserId}',[GoogleLoginController::class, 'stopGmailPushNotifications']);


Route::group(['prefix' => 'cloud-lead'], function () {
    Route::group(['prefix' => 'enrichments'], function () {
        Route::post('/', [EnrichmentController::class, 'store']);
        Route::get('/{enrichment}', [EnrichmentController::class, 'show']);
        Route::get('/list/paginated', [EnrichmentController::class, 'index']);

        Route::group(['prefix' => 'response'], function () {
            Route::post('/', [EnrichmentResponseController::class, 'store']);
        });
    });

    Route::prefix('custom-lead-requests')->group(function () {
        Route::post('/', [CustomLeadRequestController::class, 'store']); // Create a lead request
        Route::get('/{leadRequest}', [CustomLeadRequestController::class, 'show']); // Get a single lead request
        Route::put('/{leadRequest}', [CustomLeadRequestController::class, 'update']); // Update a lead request
        Route::delete('/{leadRequest}', [CustomLeadRequestController::class, 'destroy']); // Delete a lead request
        Route::get('/list/paginated', [CustomLeadRequestController::class, 'index']);

        Route::group(['prefix' => 'response'], function () {
            Route::post('/', [CustomLeadRequestResponseController::class, 'store']);
        });
    });
});
